# -*- coding: utf-8 -*- 
# -*- coding: utf-8 -*- 
########################################################################
# Module    : kzXMLSentence
# Purpose   : Manage an XML DOM tree, mostly used for generating the web
#             pages of applications.
# Author    : H. Turgut Uyar
# Copyright : ANKA Bilgi Teknolojileri, 2002
# License   : GNU Public License
#
# Notes     :
#   * To work properly with non-ASCII encodings, set the default encoding
#     in the sitecustomize.py script
#-----------------------------------------------------------------------
#  Date     : Version        Developer         Changes
#-----------------------------------------------------------------------
# 04/10/2002                Evrim zelik     self.scanTree and 
#                                             self.findFirstNode modules
#                                             are added    
#
########################################################################

import string
import sys

class kzXMLSentence:
  
  def __init__(self):
    
    self.xml    = ("",{},[],None)
    self.parent = self.xml
    
    
  def setParent(self,newParent):
    # change the parent so it will not have to specified everytime
    # append is called
    self.parent = newParent
  

  def append(self, tagName, attributes = None, text = None):
    # append an element to the XML tree at a specified location
    #   tagName: name of tag to be appended
    #   attributes: attributes of tag to be appended
    #   parentNode: the node to which the new node will be appended as a child
    #   text: the text between the opening and closing tags of the new node
    # returns the newly appended node
    
    if attributes == None:
      attributes = {}
    
    if type(attributes) != type({}):
      raise "kzXMLSentence.append : attribute type is not dictionary type"

    newNode = (tagName,attributes,[],None)
    
    if text:
      newNode[2].append(text)
    
    self.parent[2].append(newNode)
    
    return newNode
  
  def printOut(self, encoding = 'utf-8'):
    # convert the tree to a string, encode it and return it
    printed = "" #'<?xml version="1.0" ?>'
    printed = self.scanTree(self.xml[2][0],printed)
    return printed
  
    
  def scanTree(self,node,printed):
    # Scans the tree with preorder method recursively and creates the html code  
    name,attributes,children,spare = node
    printed = printed + '<' + name + ' ' 
    
    for attr in attributes.keys():
        printed = printed  + attr + '="' + attributes[attr] + '" '

    if children is None:
      printed = printed + "/>"
    else:        
      printed = printed + ">"
      for child in children:
        if type(child) == type( () ):
          printed = self.scanTree(child,printed)
        else:
          printed = printed + child
            
      printed = printed + "</" + name + ">"  
      
    return printed  
    
    
    
if __name__ == "__main__":
  # Sample usage of the class
  xmltest = kzXMLSentence()
  div1 = xmltest.append('div')
  xmltest.setParent(div1)
  xmltest.append('p', text = 'This is a test ')
  table1 = xmltest.append('table', {'class':'grid','align':'center','border':'1'})
  xmltest.setParent(table1)
  tr1 = xmltest.append('tr')
  xmltest.append('td', parentNode = tr1, text = 'Cell content')
  xmltest.setParent(div1)
  xmltest.append('br')
  xmltest.append('tr', {'valign':'top'}, table1)
  print xmltest.printOut()
  i = raw_input()
