# -*- coding: utf-8 -*- 
########################################################################
#  Proje Ad          :Kuzgun Veritabani Yonetim Sistemi
#  Modul Ad          :Form Generator
#  Hedefleri         :Bir grup tablo zerinde islemler yapmaya yarar
#  Gelitiren         :Evrim Ozcelik - evrim@ankabt.com
#  Copyright         :ANKA Bilgi Teknolojileri 2004
#  Kullanm Haklar    :GPL
#-----------------------------------------------------------------------
#  TARIH              : Versiyon        Gelistiren         Degisiklikler

########################################################################

from OFS.SimpleItem      import SimpleItem
from Globals             import DTMLFile
from OFS.ObjectManager   import ObjectManager

from AccessControl.Role  import RoleManager
from OFS.SimpleItem      import Item
from OFS.PropertyManager import PropertyManager
from OFS.FindSupport     import FindSupport
from types               import UnicodeType

from DateTime            import DateTime

import time
import string
import copy

from Products.Kuzgun.kzString      import kzString
from Products.Kuzgun.kzSort        import kzSort
from OFS.FindSupport               import FindSupport

from kzObjects    import *
from kzGroups     import *
from kzFields     import *
from kzTables     import *
from kzParameters import *
from kzScripts    import *
from kzSelections import *
from kzTableTransitions import *
from kzTemplates  import *
from kzJavascript import *


addProductForm=DTMLFile('dtml/kzDBDesignAdd', globals())

   
class kzDBDesign(PropertyManager,SimpleItem,ObjectManager):
  "kzDBDesign nesnesi"
        
  meta_type="Kuzgun DBDesign"

  _properties = (
                 {'id':'title'  , 'type':'string', 'mode':'w'},                 
                 {'id':'qrefID' , 'type':'string', 'mode':'r'},
                 {'id':'qdataID', 'type':'string', 'mode':'r'},

  )

  
  manage_main=DTMLFile('dtml/kzDBDesignMain',  globals() )
  
  __ac_permissions__=(
                       ('Change Database Methods', 
                        ('manage_main', 'manage_main')),
    
  )
   
  

  manage_options=(
    PropertyManager.manage_options
    +RoleManager.manage_options
    +Item.manage_options
    +FindSupport.manage_options            
  )

        
  def __init__(self,id):
    " DBDesign product'�zellikleri"
    self.id		    = id
    self.title              = ""

    self.qref               = ""
    self.qdata              = ""
    self.qrefID             = ""
    self.qdataID            = ""
    
                      
  def manage_editAction(self, qref, qdata, RESPONSE=None):
    " Product �liklerini g�emeye yarar"
    
    if (id=="") or (qref=="") or (qdata==""):
      RESPONSE.redirect('manage_main')

    else:
      self.qref  = qref
      self.qdata = qdata

      
#########################################################################
########################### PUBLIC METHODS ##############################
#########################################################################

  def getObjectAttributes(self,tagid):
      if tagid:
	  attr = self.qref.getAttributes(tagid)
	  obj = get_object(self.qref,self.qdata,attr,self)
	  dic = {"tagid":tagid}
	  dic.update(obj.attr)
	  return dic
      else:
	  return {}
      
  def getObjectChilds(self,tagid,tagname=""):
      if tagid:
	  return self.qref.getChilds(tagid,tagname)
      else:
	  return []
      
  def isMasterTableField(self,tagid):
    # Alanin master tabloya bagli bir alan olup olunmamasina bakiliyor
    if tagid:
	tableID = get_parent(tagid)
	master_table = self.qref.getAttributes(tableID)
	#if master_table["type"] == "s":
	#  tableID = get_parent(master_table["tagid"])
	#  master_table = self.qref.getAttributes(tableID)
	if master_table["type"] == "m":
	  return 1
	else:
	  return 0   
    else:
	return 0
    
  def gettagids(self,node,type="",list=[]):
    " A�aki gruplar�i k�ra g�getirir"
    
    if node["tagname"] == "group":

      if type == "":
        list.append((node["tagid"],node["title"]))

      elif node["isleaf"] == type:
        list.append((node["tagid"],node["title"]))
        
      if node["isleaf"] == "F":
        l_node = self.qref.getChilds(node["tagid"])
        
        for l in l_node:
          self.gettagids(l,type,list)
          
    return list
  

#########################################################################
########################### PROCESS #####################################
#########################################################################


  def process(self,request,container=None): 
    " Yap�istenen i�lemi ger�ler"
    
    buttonname = request.get("buttonname","")
    selected   = request.get("data_selected","")
    
    if buttonname and selected:
    
	if buttonname == "archive":
	    self.checkGroupForArchive(selected)
	    
	elif buttonname == "createdb":
	  if selected == "g0":
	    self.createDBfromRef(nodeID=selected,container=container)
	  else:
	    parentID = get_parent(selected)
	    self.createDBfromRef(parentID=parentID,nodeID=selected,container=container)
	    
	elif buttonname in ["newGroup","newTable","newField"]:
	    request.set("newrecord","1")
	    
	elif buttonname == 'addNewGroup':
	    dic = {'tagname' : "group",
		   'order'   : self.getOrder(selected,"group"),
		   'title'   : request.get('tree_kztitle','No Title'),
		   'isleaf'  : request.get('tree_kzisleaf','F'),
		   'activate': request.get('tree_kzactivate','1')}
	    self.newNode(selected,dic)
	    
	elif buttonname == 'saveGroupAttr':
	    dic = {'title'        : request.get('kztitle','No Title'),
		   'alias'        : request.get('kzalias',''),
		   'activate'     : request.get('kzactivate','1'),
		   'description'  : request.get('kzdescription',''),
		   'defaultconds' : request.get('kzdefaultconds',''),
		   'defaultconsts': request.get('kzdefaultconsts',''),
		   'defaultgroup': request.get('kzdefaultgroup','')}
	    self.updateNode(selected,dic)
	    
	elif buttonname in ['deleteLeafGroup','deleteGroup','deleteTable','deleteField']:
	    if selected != "g0":
		parentid  = get_parent(selected)
		attrs     = self.getObjectAttributes(selected)
		self.deleteNode(selected)
		if attrs.get("tagname","") == "table":
		    self.modifyMasterTable_tableDelete(selected)
		elif attrs.get("tagname","") == "field":
		    self.modifyMasterTable_fieldDelete(selected)   
		request.set("data_selected",parentid)
		
		
	elif buttonname == 'addNewTable':
	    dic = {'tagname'       : "table",
		   'order'         : self.getOrder(selected,"table"),
		   'title'         : request.get('tree_kztitle','No Title'),
		   'type'          : request.get('tree_kztype','1'),
		   'life'          : request.get('tree_life','365'),
		   'mastersummary' : '0'}
	    self.newNode(selected,dic)
	    
	elif buttonname == 'createMasterTable':
	    self.createMasterTable(selected)
	    
	elif buttonname == 'saveTableAttr':
	    formbuttons = request.get('kzformbuttons','')
	    designedformbuttons = request.get('kzdesignedformbuttons','')
	    formbuttonslist = formbuttons.split(",") + designedformbuttons.split(",")
	    formbuttons = ",".join(formbuttonslist)
	    dic = {'title'        : request.get('kztitle','No Title'),
		   'life'         : request.get('kzlife','365'),
		   'description'  : request.get('kzdescription',''),
		   'enable'       : request.has_key('kzenable'),
		   'style'        : request.get('kzstyle',''),
		   'mastersummary': request.has_key('kzmastersummary'),
		   'formbuttons'  : formbuttons}
	    self.updateNode(selected,dic)
	    
	elif buttonname == 'addNewField':
	    dic = {'tagname'    : "field",
		   'order'      : self.getOrder(selected,"field"),
		   'title'      : request.get("tree_kztitle",'No Title'),
		   'type'       : request.get("tree_kztype","s"),
		   'length'     : request.get("tree_kzlength",10),
		   'essential'  : "0", 
		   'indexed'    : "0", 
		   'required'   : "0",
		   'quickview'  : "1",
		   'separator'  : "0",
		   'readonly'   : "0",
		   'enable'     : "1",
		   'source'     : "default", }
	    self.newNode(selected,dic)
	    
	elif buttonname == 'saveFieldAttr':
	    dic = {'title'        : request.get("kztitle",'No Title'),
		   'length'       : request.get("kzlength",10),
		   'essential'    : request.has_key('kzessential'),
		   'enable'       : request.has_key('kzenable'),
		   'indexed'      : request.has_key('kzindexed'),
		   'required'     : request.has_key('kzrequired'),
		   'quickview'    : request.has_key('kzquickview'),
		   'readonly'     : request.has_key('kzreadonly'),
		   'separator'    : request.has_key('kzseparator'),
		   'max'          : request.get('kzmax',''),
		   'min'          : request.get('kzmin',''),
		   'default'      : request.get('kzdefault',''),
		   'source'       : request.get('kzsource',''),
		   'description'  : request.get('kzdescription',''),  
		   'separatortext': request.get('kzseparatortext',''),  
		   'parts'        : request.get('kzparts',()),
		   'widget'       : request.get('kzwidget',''),  
		   'backstyle'    : request.get('kzbackstyle',''),  
		   'fieldstyle'   : request.get('kzfieldstyle','')  }
	    self.updateNode(selected,dic)
	    self.modifyMasterTable_fieldUpdate(selected)

		
        elif buttonname in ['up','down']:
	    if selected != "g0":
		self.changeOrder(selected,buttonname)
		
	elif buttonname in ['copyGroup','copyLeafGroup','copyTable']:
	    parentid = get_parent(selected)
	    self.copyNode(parentid,selected)
	    
	elif buttonname == 'cutTable':
	    parentid = request.get("buttonvalue","")
	    parentattr = self.getObjectAttributes(parentid)
	    if parentattr.get("tagname","") == "group" and \
	       parentattr.get("isleaf","") == "T":
		self.copyNode(parentid,selected)
                
        elif buttonname == 'cutGroup':
	    parentid = request.get("buttonvalue","")
	    parentattr = self.getObjectAttributes(parentid)
	    if parentattr.get("tagname","") == "group" and \
	       parentattr.get("isleaf","") == "F":
		self.copyNode(parentid,selected)
		
	elif buttonname == 'left':
	    attrs = self.qref.getAttributes(selected) 
	    if attrs.get("tagname","") == "group":
		if selected != "g0":
		    parentid  = get_parent(selected)
		    if parentid != "g0":
			parentid  = get_parent(parentid)
			self.copyNode(parentid,selected)
		                
        elif buttonname == 'right':
	    pass
	
	elif buttonname == "archive":
	    pass

                



	    
	    
########################################################################
########################### GENERAL DESIGN METHODS #####################
########################################################################  

  def getOrder(self,parentNode,tagName):
      " Parent node altina tagname icin bir order uretir ve dondurur"
      maxOrder = 0
      for child in self.qref.getChilds(parentNode,tagName):
	  if int(child["order"]) > maxOrder:
	      maxOrder = int(child.get("order",0))
      return maxOrder + 1
  
  def newNode(self,parentNode,dic):
      " Parent node altina yeni bir obje ekler"
      obj = get_object(self.qref,self.qdata,dic,self)
      tagid = obj.insert(parentNode)
      return obj
  
  def updateNode(self,nodeID,dic):
      " Belirtilen node'un belli �likleri g�enir"
      node = self.qref.getAttributes(nodeID) 
      obj  = get_object(self.qref,self.qdata,node,self)    
      obj.update(dic)
      return obj
  
  def deleteNode(self,nodeID):
      " Belirtilen node'u siler"
      node = self.qref.getAttributes(nodeID) 
      obj  = get_object(self.qref,self.qdata,node,self)
      obj.delete()
      
  def changeOrder(self,node,direction):
    " Belirtilen node'un sirasini istenilen sekilde degistirir"
    masterid = get_parent(node)
    list     = []
    for child in self.qref.getChilds(masterid):
	if child.has_key("order"):
	    list.append({"id":child["tagid"],"order":child["order"]})
    kzSort( [list,("order","max")] )
    for i in range( len(list) ):
	if list[i]["id"] == node:
	    id1    = list[i]["id"]
	    order1 = list[i]["order"]
	    # Ust siraya cikarilmak isteniyorsa
	    if direction == "up":
		if i != 0:
		    id2    = list[i-1]["id"]
		    order2 = list[i-1]["order"]
		    control=1
		else:
		    control=0
	    # Alt siraya indirilmek isteniyorsa	    
	    else:
		if i != (len(list)-1):
		    id2    = list[i+1]["id"]
		    order2 = list[i+1]["order"]
		    control=1
		else:
		    control=0
	    break
	    
    if control:
	# D�rin yerlerini degistir                
	self.updateNode(id1,{"order":order2})        
	self.updateNode(id2,{"order":order1})
	
	
  def copyNode(self,parentID,nodeID):
      " Node'u parent altina kopyala"
      node = self.qref.getAttributes(nodeID) 
      obj = get_object(self.qref,self.qdata,node,self)
      new_tagid = obj.copy(parentID)  # hata: Zope Kopyalarken kilitleniyor 
      return new_tagid
  
  def gettagids(self,node,type="",list=[]):
      " Agactaki gruplari belli kisitlara g�getirir"
      if node["tagname"] == "group":
	if type == "":
	  list.append((node["tagid"],node["title"]))
	elif node["isleaf"] == type:
	  list.append((node["tagid"],node["title"]))
  
	if node["isleaf"] == "F":
	  l_node = self.qref.getChilds(node["tagid"])
	  for l in l_node:
	    self.gettagids(l,type,list)
      return list
    

  def copy_childs(self,parent1,parent2):
      " parent1'in �uklarn parent2'ye kopyala"
      for child in self.qref.getChilds(parent1):   
	  self.qref.copy(child['tagid'],parent2)
        
        
  def createMasterTable(self,groupID):
      " Se�i grup i�de master tablo olusturur"
	      
      if len(self.qref.getChilds(groupID,"table","*",["type","==","m"])):
	  return        
      
      groupattr = self.qref.getAttributes(groupID)
      title   = groupattr.get("title","")
      
      # Grup i�de master tablo yarat
      order = self.getOrder(groupID,"table")
      dic   = {}
      dic['tagname']      = "table"
      dic['order']        = order
      dic['title']        = title 
      dic['type']         = "m"
      dic['life']         = "365"

      masterID = self.newNode(groupID,dic).tagid
      
      # Grubun altindaki t�lolari tara
      for table in self.qref.getChilds(groupID,"table"):
	  if table["type"] != "m":

	      # Tablonun t�manlarini tara
	      for node in self.qref.getChilds(table["tagid"],"field"):
		  # Eger alan zorunlu bir alansa
		  if node["essential"] == "1":
		      temp  = copy.deepcopy(node)
		      order = self.getOrder(masterID,"field")
		      temp["order"]     = str(order)
		      temp["reference"] = node["tagid"][len(groupID):]
		      newobj = self.newNode(masterID,temp) 
		      new_tagid = newobj.tagid
		      self.copy_childs(node["tagid"],new_tagid)
		      # Kopyasi olu�turulan alanin �likleri degistiriliyor
		      self.updateNode(node["tagid"],
				      {"reference":new_tagid[len(groupID):]})
						      
		  # E�d�lt tablo ise
		  elif node["tagname"] == "table":
		      for node2 in self.qref.getChilds(node["tagid"],"field"):
			  # Eger alan zorunlu bir alansa
			  if node2["essential"] == "1": 
			      temp  = copy.deepcopy(node2)
			      order = self.getOrder(masterID,"field")
			      temp["order"]     = str(order)
			      temp["reference"] = node2["tagid"][len(groupID):]
			      newobj = self.newNode(masterID,temp) 
			      new_tagid = newobj.tagid
			      self.copy_childs(node2["tagid"],new_tagid)
			      # Kopyasi olusturulan alanin �likleri degistiriliyor
			      self.updateNode(node2["tagid"],
					      {"reference":new_tagid[len(groupID):]})
			      
                                        
                                        
  def findMasterTable(self,node_id):
      " Icinde bulunulan grubun master tablosunu getirir"

      node = self.qref.getAttributes(node_id) 
      # Icinde bulunulan yaprak gruba ulasiliyor
      if node["tagname"] == "group":
	  if node["isleaf"] == "F":
	      return (-1,-1)
      else:
	  tag = node["tagname"]
	  while(tag!="group"):
	      id       = node["tagid"]
	      index    = string.rfind(id,"_")
	      masterid = id[0:index]
	      node     = self.qref.getAttributes(masterid) 
	      tag      = node["tagname"]
      rec = self.qref.getChilds(node["tagid"],"table","*",["type","==","m"])        
      if len(rec):
	  return node["tagid"],rec[0]["tagid"]
      else:
	  return (node["tagid"],-1) 
        
    
    
  def modifyMasterTable_tableDelete(self,tableid):
      " Tablo silindiginde master tabloyu de�irir"
      # Master tabloysa de�klik yapma
      tableattr = self.qref.getAttributes(tableid)
      if tableattr["type"] == "m":
	  return

      groupID,tableID = self.findMasterTable(tableid)
      if tableID == -1:
	  return

      for node in self.qref.getChilds(tableid):
	  if node["tagname"] == "field":
	      if node["essential"] in [1,"1"]:
		  refID = node["reference"]
		  self.deleteNode(groupID+refID)
	  elif node["tagname"] == "table":
	      for node2 in self.qref.getChilds(node["tagid"]):
		  if node2["essential"] in [1,"1"]:
		      refID = node2["reference"]
		      self.deleteNode(groupID+refID)
                        
                    
                
  def modifyMasterTable_fieldDelete(self,fieldid):
      " Alan silindiginde master tabloyu degistirir"
      
      groupID,tableID = self.findMasterTable(fieldid)
      if tableID == -1:
	  return
      
      fieldattr = self.qref.getAttributes(fieldid)
      table = get_parent(fieldid)
      
      # E�master tablonun alan�     
      if tableID == table:
	  if fieldattr["reference"] != "":
	      refID = fieldattr["reference"]
	      self.updateNode(groupID+refID,
			      {"reference":"",
			       "essential":"0"})
      
      elif fieldattr["essential"] in [1,"1"]:
	  refID = fieldattr["reference"]
	  self.deleteNode(groupID+refID)
            
  
  def modifyMasterTable_fieldUpdate(self,fieldid):
      " Alan g�endi�e master tabloyu de�irir"
      
      groupID,masterID = self.findMasterTable(fieldid)  
      if masterID == -1:
	  return
      
      tableID = get_parent(fieldid)
      
      # Referanstaki �likleri
      node    = self.qref.getAttributes(fieldid)
      
      # Master tablonun zorunlu alan�nmi�se referans�lt
      if masterID == tableID:
	  return
	    
      # Zorunlu alansa        
      elif node["essential"] in [1,"1"]:
	  # Daha �den zorunlu alan de�e
	  if node["reference"] == "":
	       temp = copy.deepcopy(node)
	       order = self.getOrder(masterID,"field")
	       temp["order"]     = str(order)
	       temp["essential"] = "1"
	       temp["reference"] = node["tagid"][len(groupID):]
	       newobj = self.newNode(masterID,temp) 
	       new_tagid = newobj.tagid
	       self.copy_childs(node["tagid"],new_tagid)
	       # Kopyas�turulan alan�llikleri de�iriliyor
	       self.updateNode(node["tagid"],
			       {"reference":new_tagid[len(groupID):]})
	      
      # Zorunlu alan olmaktan �m�      
      elif node["essential"] in [0,"0"]:
	  # Daha �den zorunlu alansa
	  if node["reference"] != "":
	      refID = node["reference"]
	      self.deleteNode(groupID+refID)
	      self.updateNode(node["tagid"],
			      {"reference":""})
		  
  
    
########################################################################
###########################  FORMS #####################################
########################################################################  

  def form_process(self,REQUEST):
      # Belirtilen bir form islem yapilir
      buttonname  =  REQUEST.get('buttonname','')
      REQUEST.set('buttonname','')
      if buttonname == "new":
          REQUEST.set('newform','1')
          REQUEST.set('changeform','0')
      elif buttonname == "add":
          self.add_select(REQUEST)
          REQUEST.set('newform','0')
          REQUEST.set('changeform','0')
      elif buttonname == "change":
          REQUEST.set('newform','0')
          REQUEST.set('changeform','1')
      elif buttonname == "save":
          self.save_select(REQUEST)
          REQUEST.set('newform','0')
          REQUEST.set('changeform','0')    
      elif buttonname == "delete":
          self.delete_select(REQUEST)
          REQUEST.set('newform','0')
          REQUEST.set('changeform','0')      
      else:
          REQUEST.set('newform','0')
          REQUEST.set('changeform','0')
          
      
      
  def get_formmode(self,REQUEST):
      # Belirtilen bir form
      newform    =  REQUEST.get('newform','')
      changeform =  REQUEST.get('changeform','')
      if newform == "1":
          return "new"
      elif changeform == "1":
          return "change"
      else:
          return "print"

  def gettable_forms(self,REQUEST):
      # Tablo icin kullanilacak formlar alinir
      parentid = REQUEST.get('parentid','')
      if parentid == '':
          return []
      else:
          return self.qref.getChilds(parentid,'form')
      
  def get_form(self,REQUEST):
      # Belirtilen bir formun degeri alinir
      formmode = self.get_formmode(REQUEST)
      buttonvalue =  REQUEST.get('buttonvalue','')
      if formmode == 'new':
          return {}
      
      elif formmode == 'change':
          if buttonvalue != "":
              return self.qref.getAttributes(buttonvalue)
          else:
              return {}
      
      else:
          return {}
      
  def get_formFields(self,parentid):
      return self.qref.getChilds(parentid,"field")
      
     
  def add_form(self,REQUEST):
      # Tabloya ge��ekler
      temp_dic = {}
      temp_dic['tagname']   = 'form'
      temp_dic['row']  = REQUEST.get('row','0')
      temp_dic['col']  = REQUEST.get('col','0')
      selectobj = get_object(self.qref,self.qdata,temp_dic,self)
      parentid = REQUEST.get('parentid','')
      if parentid != "":
          selectobj.insert(parentid)  
      
      
  def save_form(self,REQUEST):
      # Tablodaki ge�� g�celler
      temp_dic = {}
      temp_dic['tagname']   = 'select'
      temp_dic['tagid']     = REQUEST.get('tagid','')
      temp_dic['selectid']  = REQUEST.get('selectid','')
      temp_dic['text']      = REQUEST.get('text','')
      selectobj = get_object(self.qref,self.qdata,temp_dic,self)
      selectobj.update(temp_dic)  
  
  def delete_form(self,REQUEST):
      # Tablodaki secimleri siler
      temp_dic = {}
      temp_dic['tagname']  = 'select'
      selectIds            = REQUEST.get('tagid','')
      for selectId in selectIds:
          temp_dic['tagid']  = selectId
          selectobj = get_object(self.qref,self.qdata,temp_dic,self)
          selectobj.delete()  
    
            
########################################################################
###########################  FORMBUTTON ################################
########################################################################

  def formbutton_process(self,REQUEST):
    # Form dugmeleri islemi yapilir
    buttonname  =  REQUEST.get('buttonname','')
    REQUEST.set('buttonname','')
    if buttonname == "new":
          REQUEST.set('kzmode','new')
    elif buttonname == "add":
	id = self.add_formbutton(REQUEST)
	return
	if id is None:
	    REQUEST.set('kzmode','print')
	else:
	    REQUEST.set('tempid',id)
	    REQUEST.set('kzmode','edit')
    elif buttonname == "change":
	REQUEST.set('kzmode','edit')
    elif buttonname == "save":
	self.save_template(REQUEST)
	REQUEST.set('kzmode','edit') 
    elif buttonname == "delete":
	self.delete_template(REQUEST)
	REQUEST.set('kzmode','print')  
    elif buttonname == "view":
	REQUEST.set('kzmode','view')      
    else:
	REQUEST.set('kzmode','print')
	  
	  
  def gettable_formbuttons(self,REQUEST):    
    parentid = REQUEST.get('parentid','')
    result = []
    if parentid == '':
      return result
    else:
      container = self.qref.getContainer(parentid)
      temp = FindSupport.ZopeFind(FindSupport(container), container, [],
				  ['kzFormButton'],search_sub=0) 
      for t in temp:
	#href = "parentid=%s&parent_tagname=%s&tagid=%s&buttonname=change" % (parentid,'table',parentid+'_'+t[0])
	href = t[1].absolute_url()
	result.append( {'id':t[0],'title':getattr(t[1],'title','notitle'),
			'type':t[1].meta_type,'idhref':(href,0),'titlehref':(href,0)} )
      return result 
    
  def get_formbutton(self,REQUEST):
    tagid = REQUEST.get('tagid','')
    if tagid:
      object = self.qref.getContainer(tagid)
      return object.formButtonEdit()
    else:
      return {}
    
  def add_formbutton(self,REQUEST):
      # Tabloya sablonu ekler
      parentid = REQUEST.get('parentid','')
      title    = REQUEST.get('title','')
      if parentid != '':
	container = self.qref.getContainer(parentid)
	datetime = DateTime()
	year,month,day,hour,minute,second = datetime.year(),datetime.month(),datetime.day(),\
	    datetime.hour(), datetime.minute(),datetime.second()
	# Eklenecek button'un id'si 
	id = str("kzFormButton%s%s%s%s" % (year,month,day,str(hour)+str(minute)+str(second))).replace(".","")
	container.invokeFactory(id=id,type_name='kzFormButton')
      else:
	return None

    
########################################################################
###########################  TEMPLATES #################################
########################################################################

  def get_tempview(self,REQUEST):
      tagid = REQUEST.get('tempid','')
      if tagid!="":
          container = self.qref.getContainer(tagid)
          return container.view()


  def template_process(self,REQUEST):
      # Sablon islemi yapilir
      buttonname  =  REQUEST.get('buttonname','')
      REQUEST.set('buttonname','')
      if buttonname == "new":
          REQUEST.set('kzmode','new')
      elif buttonname == "add":
          id = self.add_template(REQUEST)
          if id is None:
              REQUEST.set('kzmode','print')
          else:
              REQUEST.set('tempid',id)
              REQUEST.set('kzmode','edit')
      elif buttonname == "change":
          buttonvalue  =  REQUEST.get('buttonvalue','')
          if buttonvalue != "":
              REQUEST.set('buttonvalue','')
              parentid = REQUEST.get('parentid','')
              REQUEST.set('tempid',"%s_%s" % (parentid,buttonvalue) )
          REQUEST.set('kzmode','edit')
      elif buttonname == "save":
          self.save_template(REQUEST)
          REQUEST.set('kzmode','edit') 
      elif buttonname == "delete":
          self.delete_template(REQUEST)
          REQUEST.set('kzmode','print')  
      elif buttonname == "view":
          REQUEST.set('kzmode','view')      
      else:
          REQUEST.set('kzmode','print')
          
      
      
  def get_templatemode(self,REQUEST):
      # Sablon isleminin durumu alinir 
      tempmode =  REQUEST.get('tempmode','')
      return tempmode

  def gettable_templates(self,REQUEST):
      # Tablo icin kullanilacak gecisleri dondurur
      parentid = REQUEST.get('parentid','')
      result = []
      if parentid == '':
	return result
      else:
	container = self.qref.getContainer(parentid)
	return  FindSupport.ZopeFind(FindSupport(container), container, [],
				    ['kzTemplates'],search_sub=0) 
      
  def gettable_templates2(self,REQUEST):
      # Tablo icin kullanilacak gecisleri dondurur
      parentid = REQUEST.get('parentid','')
      result = []
      if parentid == '':
	return result
      else:
	container = self.qref.getContainer(parentid)
	temp = FindSupport.ZopeFind(FindSupport(container), container, [],
				    ['kzTemplates','kzHtmlReport','kzRmlReport'],search_sub=0) 
	for t in temp:
	  href = "parentid=%s&parent_tagname=%s&tempid=%s&buttonname=change" % (parentid,'table',parentid+'_'+t[0])
	  result.append( {'id':t[0],'title':getattr(t[1],'title','notitle'),
			  'type':t[1].meta_type,'idhref':(href,1),'titlehref':(href,1),
			  'view':'view','viewhref':(t[1].getTemplateOutputAction(),0)} )
	return result    
      
  def get_template(self,REQUEST):
      # Belirtilen bir sablon alinir
      tagid = REQUEST.get('tempid','')
      dic = {}
      tempobj = self.qref.getContainer(tagid)
      return tempobj.getTemplateData()

  def get_temppath(self,REQUEST):
      tagid = REQUEST.get('tempid','')
      if tagid!="":
          container = self.qref.getContainer(tagid)
          return container.getPhysicalPath()
      
  def get_tempurl(self,REQUEST):
      tagid = REQUEST.get('tempid','')
      if tagid!="":
          container = self.qref.getContainer(tagid)
          return container.absolute_url()    
     
  def add_template(self,REQUEST):
      # Tabloya sablonu ekler
      parentid = REQUEST.get('parentid','')
      title    = REQUEST.get('title','')
      temptype = REQUEST.get('templatetype','kzHtmlReport')
      if parentid != '':
          temp_dic = {"tagname":"templates","title":title,'templatetype':temptype}
          templateobj = get_object(self.qref,self.qdata,temp_dic,self)
          return templateobj.insert(parentid)
      else:
          return None

      
  def save_template(self,REQUEST):
      # Tablodaki sablonu gunceller
      tagid = REQUEST.get('tempid','')
      if tagid:
          temp_dic = {}
          temp_dic['tagname']      = 'templates'
          temp_dic['tagid']        = tagid
          temp_dic['title']        = REQUEST.get('title','')
          temp_dic['sysparams']    = REQUEST.get('sysparams','')
          temp_dic['templatetext'] = REQUEST.get('templatetext','')
          temp_dic['scriptparams'] = REQUEST.get('scriptparams','')
          temp_dic['scripttext']   = REQUEST.get('scripttext','')
          templateobj = get_object(self.qref,self.qdata,temp_dic,self)
          templateobj.update(temp_dic)
          self.set_templateErrors(tagid,REQUEST)
      
  def set_templateErrors(self,tagid,REQUEST):
      temp = self.qref.getContainer(tagid)
      scripterrors = temp.getScriptErrors()
      if scripterrors:
	REQUEST.set("scripterrors",scripterrors)
      talerrors = temp.getTemplateErrors()
      if talerrors:
	REQUEST.set("talerrors",str(talerrors))             
      
  def delete_template(self,REQUEST):
      # Tablodaki gecisi siler
      temp_dic = {}
      temp_dic['tagname']    = 'templates'
      templateIds            = REQUEST.get('ids',[])
      parentid               = REQUEST.get('parentid','')
      for templateId in templateIds:
          temp_dic['tagid']  = "%s_%s" % (parentid,templateId)
          templateobj = get_object(self.qref,self.qdata,temp_dic,self)
          templateobj.delete()  
    
    
    
    
    
########################################################################
###########################  SELECTIONS ################################
########################################################################

  def select_process(self,REQUEST):
      # Belirtilen bir ge��n de�ri al��
      buttonname  =  REQUEST.get('buttonname','')
      REQUEST.set('buttonname','')
      if buttonname == "new":
          REQUEST.set('newselect','1')
          REQUEST.set('changeselect','0')
      elif buttonname == "add":
          self.add_select(REQUEST)
          REQUEST.set('newselect','0')
          REQUEST.set('changeselect','0')
      elif buttonname == "change":
          REQUEST.set('newselect','0')
          REQUEST.set('changeselect','1')
      elif buttonname == "save":
          self.save_select(REQUEST)
          REQUEST.set('newselect','0')
          REQUEST.set('changeselect','0')    
      elif buttonname == "delete":
          self.delete_select(REQUEST)
          REQUEST.set('newselect','0')
          REQUEST.set('changeselect','0')      
      else:
          REQUEST.set('newselect','0')
          REQUEST.set('changeselect','0')
          
      
      
  def get_selectmode(self,REQUEST):
      # Belirtilen bir ge��n de�ri al��
      newselect    =  REQUEST.get('newselect','')
      changeselect =  REQUEST.get('changeselect','')
      if newselect == "1":
          return "new"
      elif changeselect == "1":
          return "change"
      else:
          return "print"

  def gettable_selects(self,REQUEST):
      # Tablo i�n kullan�acal ge��eri d�enler
      parentid = REQUEST.get('parentid','')
      if parentid == '':
          return []
      else:
          return self.qref.getChilds(parentid,'select')
      
  def get_select(self,REQUEST):
      # Belirtilen bir ge��n de�ri al��
      selectmode = self.get_selectmode(REQUEST)
      buttonvalue =  REQUEST.get('buttonvalue','')
      if selectmode == 'new':
          return {}
      
      elif selectmode == 'change':
          if buttonvalue != "":
              return self.qref.getAttributes(buttonvalue)
          else:
              return {}
      
      else:
          return {}
      
     
  def add_select(self,REQUEST):
      # Tabloya ge��ekler
      temp_dic = {}
      temp_dic['tagname']   = 'select'
      temp_dic['selectid']  = REQUEST.get('selectid','')
      temp_dic['text']      = REQUEST.get('text','')
      selectobj = get_object(self.qref,self.qdata,temp_dic,self)
      parentid = REQUEST.get('parentid','')
      if parentid != "":
          selectobj.insert(parentid)  
      
      
  def save_select(self,REQUEST):
      # Tablodaki ge�� g�celler
      temp_dic = {}
      temp_dic['tagname']   = 'select'
      temp_dic['tagid']     = REQUEST.get('tagid','')
      temp_dic['selectid']  = REQUEST.get('selectid','')
      temp_dic['text']      = REQUEST.get('text','')
      selectobj = get_object(self.qref,self.qdata,temp_dic)
      selectobj.update(temp_dic)  
  
  def delete_select(self,REQUEST):
      # Tablodaki secimleri siler
      temp_dic = {}
      temp_dic['tagname']  = 'select'
      selectIds            = REQUEST.get('tagid','')
      for selectId in selectIds:
          temp_dic['tagid']  = selectId
          selectobj = get_object(self.qref,self.qdata,temp_dic)
          selectobj.delete()  
    

########################################################################
###########################  JAVA SCRIPTS ##############################
########################################################################

  def javascript_process(self,REQUEST):
      # Script icin isleri duzenler
      buttonname     = REQUEST.get('buttonname','')
      parentid       = REQUEST.get('parentid','')
      tagname        = REQUEST.get('tagname','')
      tagid          = "%s_%s" % (parentid,tagname)
      const_param    = ""
      
      if buttonname == 'save':
          self.save_javaScript(tagid,tagname,REQUEST,const_param)    
      elif buttonname == 'delete':
          self.delete_javaScript(tagid,REQUEST)
      elif buttonname == 'runscript':
          REQUEST.set('runscript','1')
         
      tagname = REQUEST.get('tagname','')
      if tagname:    
          return self.get_javascript(parentid,tagname,const_param)
      else:
          return {}
  

  def get_javascripts(self,parentid):
      # Javascript listesi dondurur
      if parentid == '': return []
      else: return self.qref.getChilds(parentid,'javascript')

      
  def get_javascript(self,parentid,tagname,const_param):
      tagid = "%s_%s" % (parentid,tagname)
      dic   = {'call':'','params':const_param,'text':''}
      if tagid != '':
          try: 
              script = self.qref.getContainer(tagid)
          except: 
              self.add_javaScript(parentid,tagname,const_param)
              script = self.qref.getContainer(tagid)
          if script is not None:
              dic['call']   = getattr(script,'call')
              dic['params'] = getattr(script,'title')
              dic['text']   = getattr(script,'raw')
      return dic  
      

  def add_javaScript(self,parentid,tagname,const_param): 
      dic = {'call':const_param,'params':const_param,'text':'','tagname':tagname,'tagid':tagname}
      scriptobj = get_object(self.qref,self.qdata,dic)
      scriptobj.insert(parentid)
      
      
  def save_javaScript(self,tagid,tagname,REQUEST,const_param):
      params  = self.control_scriptParameters(const_param,REQUEST.get('params',''))
      params  = self.control_triggerCall(params)
      call    = self.control_scriptParameters(const_param,REQUEST.get('call',''))
      call    = self.control_triggerCall(call)
      if tagid != '':
          dic = {'call'   :call,
                 'params' :params,
                 'text'   :REQUEST.get('text',''),
                 'tagid'  :tagid,
                 'tagname':tagname}
          scriptobj = get_object(self.qref,self.qdata,dic)
          scriptobj.update(dic)
          
       
  def delete_javaScript(self,tagid,REQUEST):
    # Tablodaki javascript'leri siler
    temp_dic = {}
    temp_dic['tagname']  = 'javascript'
    tagids            = REQUEST.get('tagid','')
    for id in tagids:
        if id == tagid: REQUEST.set("tagname","")            
        temp_dic['tagid']  = id
        scriptobj = get_object(self.qref,self.qdata,temp_dic)
        scriptobj.delete()          
          
    

          
########################################################################
###########################  SCRIPTS ###################################
########################################################################

  def script_process(self,REQUEST):
      # Script icin isleri duzenler
      buttonname     = REQUEST.get('buttonname','')
      parentid       = REQUEST.get('parentid','')
      parent_tagname = REQUEST.get('parent_tagname','')
      tagname        = REQUEST.get('tagname','')
      tagid          = "%s_%s" % (parentid,tagname)
      
      
      if tagname in ["script"]:
          const_param=""
          if buttonname == 'save':
              self.save_pythonScript(tagid,tagname,REQUEST,const_param)
      elif tagname in ["oninsert","beforeinsert","onupdate","beforeupdate","ondelete","beforedelete"]:  
          const_param="form_values"
          if buttonname == 'save':
              self.save_pythonScript(tagid,tagname,REQUEST,const_param) 
      elif tagname in ["dynamicprops"]:  
          const_param="kzobject"
          if buttonname == 'save':
              self.save_pythonScript(tagid,tagname,REQUEST,const_param) 	      
      elif tagname in ["ontime"]:
          const_param="tableid,form_values"
          if buttonname == 'save':
              self.save_timeScript(tagid,tagname,REQUEST,const_param)        

      if buttonname == 'delete':        
          self.delete_pythonScript(parentid,tagid,REQUEST)
      elif buttonname == 'runscript':
          REQUEST.set('runscript','1')
         
      tagname =  REQUEST.get('tagname','')
      if tagname:
          return self.get_script(parentid,tagname,const_param)    
      else:
          return {}
      
      
  def get_scripts(self,parentid):
    # Script listesi dondurur
    if parentid == '': 
        return []
    else: 
        container = self.qref.getContainer(parentid)
        templist = FindSupport.ZopeFind(FindSupport(container), container, [],
                                    ['Script (Python)'],search_sub=0) 
        scripts = []
        for temp in templist:
            scripts.append( {"tagid": "%s_%s" % (parentid,getattr(temp[1],"id","")),
                             "tagname":getattr(temp[1],"id",""),
                             "params":getattr(temp[1],"_params","")
                              } ) 
        return scripts
        

  
  def get_script(self,parentid,tagname,const_param):
      if tagname in ["dynamicprops","script","oninsert","beforeinsert","onupdate","beforeupdate","ondelete","beforedelete"]:
          return self.get_pythonScript(parentid,tagname,const_param)
      elif tagname == "ontime":
          return self.get_timeScript(parentid,tagname,const_param)
      
      
  def get_pythonScript(self,parentid,tagname,const_param):
      tagid = "%s_%s" % (parentid,tagname)
      dic   = {'call':'','params':const_param,'text':'','url':''}
      if tagid != '':
          try: 
              script = self.qref.getContainer(tagid)
          except: 
              self.add_pythonScript(parentid,tagname,const_param)
              script = self.qref.getContainer(tagid)
          if script is not None:
              dic['call']         = getattr(script,'call')
              dic['params']       = getattr(script,'_params')
              dic['text']         = getattr(script,'_body')
	      dic['physicalpath'] = script.getPhysicalPath()
      return dic  
  
  
  def get_timeScript(self,parentid,tagname,const_param):
      tagid = "%s_%s" % (parentid,tagname)
      dic   = {'executeAt':'','periodDays':'',
               'call':'','params':'','text':''}
      if tagid != '':
          try: 
              script = self.qref.getContainer(tagid)
              xron   = self.qref.getContainer(tagid+'xron')
          except: 
              self.add_timeScript(parentid,tagname,const_param)
              script = self.qref.getContainer(tagid)
              xron   = self.qref.getContainer(tagid+'xron')
          if script is not None:
              dic['call']   = getattr(script,'call')
              dic['params'] = getattr(script,'_params')
              dic['text']   = getattr(script,'_body')
          if xron is not None:
              date = getattr(xron,'executeAt')
              dic['executeAt']  = "%s/%s/%s %s:%s:%s" % (string.zfill(date.year(),4),
                                                         string.zfill(date.month(),2),
                                                         string.zfill(date.day(),2),
                                                         string.zfill(date.hour(),2),
                                                         string.zfill(date.minute(),2),
                                                         string.zfill(date.second(),2))
              dic['periodDays'] = getattr(xron,'periodDays')
      return dic      


  def control_scriptParameters(self,constparams="",params=""):
      if len(constparams):
          cparamlist= str(constparams).strip().split(",")
          paramlist = str(params).strip().split(",")
          list      = []
          index     = 0
          for cparam in cparamlist:
              if paramlist[index] == cparam:
                  index += 1
                  list.append(cparam)
              else:
                  break
          list = list + cparamlist[index:] + paramlist[index:]
          while "" in list:
              list.remove("")
          params = string.join(list,",")
      return params   
  
  def control_triggerCall(self,params=""):
      paramlist = str(params).strip().split(",")
      list      = []
      for param in paramlist:
          if not "|" in param:
              list.append(param)
      return string.join(list,",")    
              
      
  
  def add_pythonScript(self,parentid,tagname,const_param): 
      dic = {'call':const_param,'params':const_param,'text':'','tagname':tagname}
      scriptobj = get_object(self.qref,self.qdata,dic)
      scriptobj.insert(parentid)
      
      
  def add_timeScript(self,parentid,tagname,const_param):
      container = self.qref.getContainer(parentid)
      # Scripti ekle
      if not hasattr(container,tagname):
          self.add_pythonScript(parentid,tagname,const_param)
      # Xron ekle
      id = "%sxron" % tagname
      if not hasattr(container,'ontimexron'):
          container.manage_addProduct['Xron'].manage_addXronDTMLMethod(id,executeAt = DateTime())
          xron = getattr(container,id)
          setattr(xron,"raw","<dtml-call expr='scan_ontrigger(%s,%s)'>" % (tagname,parentid))
          

  def save_pythonScript(self,tagid,tagname,REQUEST,const_param):
      params  = self.control_scriptParameters(const_param,REQUEST.get('params',''))
      if tagname!='script':
        params  = self.control_triggerCall(params)
      call    = self.control_scriptParameters(const_param,REQUEST.get('call',''))
      if tagname!='script':
        call    = self.control_triggerCall(call)
      if tagid != '':
          dic = {'call'   :call,
                 'params' :params,
                 'text'   :REQUEST.get('text',''),
                 'tagid'  :tagid,
                 'tagname':tagname}
          scriptobj = get_object(self.qref,self.qdata,dic)
          scriptobj.update(dic)
          self.set_scriptErrors(tagid,REQUEST)
          
  def save_timeScript(self,tagid,tagname,REQUEST,const_param):
    # Save script
    self.save_pythonScript(tagid,tagname,REQUEST,const_param)
    # Save Xron
    xronid = "%sxron" % tagid
    xron   = self.qref.getContainer(xronid)
    executeAt = REQUEST.get('executeAt','')
    periodDays= REQUEST.get('periodDays',0.0)
    try: executeAt = DateTime(executeAt)
    except: executeAt = DateTime()
    REQUEST.set('executeAt',executeAt)
    REQUEST.set('periodDays',periodDays)
    data = getattr(xron,'raw')
    xron.manage_edit(data,title='',REQUEST=REQUEST)        
                  
                  
  def set_scriptErrors(self,tagid,REQUEST):
      script = self.qref.getContainer(tagid)
      errors=getattr(script,"errors",None)
      if errors is not None:
          if len(errors):
              REQUEST.set("scripterrors",str(errors))                  
          
  def get_scriptresults(self,REQUEST):
      parentid       = REQUEST.get('parentid','')
      tagname        = REQUEST.get('tagname','')
      tagid          = "%s_%s" % (parentid,tagname)
      if tagid != '':
          REQUEST.set('runscript','0')
          script = self.qref.getContainer(tagid)
          return eval('script(%s)' % (getattr(script,'call')) ) 
      
      
  def delete_pythonScript(self,parentid,tagid,REQUEST):
    # Tablodaki script'leri siler
    temp_dic = {}
    temp_dic['tagname']  = 'script'
    tagids            = REQUEST.get('tagid','')
    for id in tagids:
        if id == tagid: REQUEST.set("tagname","")            
        temp_dic['tagid']  = id
        scriptobj = get_object(self.qref,self.qdata,temp_dic)
        scriptobj.delete()  
        scriptid = tagid[len(parentid)+1:]
        if scriptid in ["ontime"]:
            container = self.qref.getContainer(parentid)
            container._delObject(scriptid)
      
      
########################################################################
###########################  PARAMETERS ################################
########################################################################

  def param_process(self,REQUEST):
      # Belirtilen parametre islemi yapilir
      buttonname  =  REQUEST.get('buttonname','')
      REQUEST.set('buttonname','')
      if buttonname == "new":
          REQUEST.set('newparam','1')
          REQUEST.set('changeparam','0')
      elif buttonname == "add":
          self.add_param(REQUEST)
          REQUEST.set('newparam','0')
          REQUEST.set('changeparam','0')
      elif buttonname == "change":
          REQUEST.set('newparam','0')
          REQUEST.set('changeparam','1')
      elif buttonname == "save":
          self.save_param(REQUEST)
          REQUEST.set('newparam','0')
          REQUEST.set('changeparam','0')    
      elif buttonname == "delete":
          self.delete_param(REQUEST)
          REQUEST.set('newparam','0')
          REQUEST.set('changeparam','0')      
      else:
          REQUEST.set('newparam','0')
          REQUEST.set('changeparam','0')
          
      
      
  def get_parammode(self,REQUEST):
      # Belirtilen bir ge��n de�ri al��
      newparam    =  REQUEST.get('newparam','')
      changeparam =  REQUEST.get('changeparam','')
      if newparam == "1":
          return "new"
      elif changeparam == "1":
          return "change"
      else:
          return "print"

  def gettable_params(self,parentid):
      # Tablo i�n kullan�acal gecisleri d�enler
      if parentid == '': return []
      else: return self.qref.getChilds(parentid,'param')
      
  def get_param(self,REQUEST):
      # Belirtilen bir ge��n de�ri al��
      parammode = self.get_parammode(REQUEST)
      buttonvalue =  REQUEST.get('buttonvalue','')
      if parammode == 'new':
          return {}
      
      elif parammode == 'change':
          if buttonvalue != "":
              return self.qref.getAttributes(buttonvalue)
          else:
              return {}
      
      else:
          return {}
      
  def get_table_field(self,tagid):
      # Tablo ve alan isimlerini d�d�
      if tagid=="":
          return ({"title":""},{"title":""})
      else:
          try: fieldAttr = self.qref.getAttributes(tagid)
          except: fieldAttr = {"title":self.getText("Deleted") }
          # Parametrenin g�rdi�ablo ba�l�lunuyor 
          tagid = get_parent(tagid)
          try: tableAttr = self.qref.getAttributes(tagid)
          except: tableAttr = {"title":self.getText("Deleted")}
          return (tableAttr,fieldAttr)
  
      
  def add_param(self,REQUEST):
      # Tabloya ge��ekler
      temp_dic = {}
      temp_dic['tagname']        = 'param'
      temp_dic['paramid']        = REQUEST.get('paramid','')
      temp_dic['fieldrelation']  = REQUEST.get('fieldrelation','')
      temp_dic['paramtype']      = REQUEST.get('paramtype','2')
      paramobj = get_object(self.qref,self.qdata,temp_dic)
      parentid = REQUEST.get('parentid','')
      if parentid != "":
          paramobj.insert(parentid)  
      
      
  def save_param(self,REQUEST):
      # Tablodaki ge�� g�celler
      temp_dic = {}
      temp_dic['tagid']          = REQUEST.get('tagid','')
      temp_dic['tagname']        = 'param'
      temp_dic['paramid']        = REQUEST.get('paramid','')
      temp_dic['fieldrelation']  = REQUEST.get('fieldrelation','')
      temp_dic['paramtype']      = REQUEST.get('paramtype','2')
      paramobj = get_object(self.qref,self.qdata,temp_dic)
      paramobj.update(temp_dic)  
  
  def delete_param(self,REQUEST):
      # Tablodaki ge�� siler
      temp_dic = {}
      temp_dic['tagname']  = 'param'
      paramIds             = REQUEST.get('tagid','')
      for paramId in paramIds:
          temp_dic['tagid']  = paramId
          paramobj = get_object(self.qref,self.qdata,temp_dic)
          paramobj.delete()  
    
          
########################################################################
######################## PARAMETER INFO ################################
########################################################################
          
  def get_field_db_attr(self,paramid,fieldid):
      # Alan� id'sinden �elli�ne bakarak veri tabaninda hangi durumda bulunaca�n�s�ler
      from Products.kzFormGenerator.kzFields import address_parts,telephone_parts,\
           integer_parts,money_parts,date_parts,picture_parts
      try:
          dic = {"i":address_parts,"t":telephone_parts,"i":integer_parts,
                 "m":money_parts,"d":date_parts,"p":picture_parts}
          # Alan silinmi�olabilir
          field = self.qref.getAttributes(fieldid)
          if field['type'] in ["s","l","v","w","e"]:
              return [("%s" % paramid,field["title"])]
          
          else:
              list = []
              i    = 0
              for key,length in dic.get(field['type'],[]):
                  list.append( ("%s+'_%s'" % (paramid,i) , self.getText(key) ) )
                  i = i + 1
                  
              return list
      except:
          return [("%s" % paramid , self.getText('Corrupted'))]
          
          
          
                    
    
########################################################################
######################## TABLE TRASITIONS ##############################
########################################################################

  def trans_process(self,REQUEST):
      # Belirtilen bir ge��n de�ri al��
      buttonname  =  REQUEST.get('buttonname','')
      REQUEST.set('buttonname','')
      if buttonname == "new":
          REQUEST.set('newtrans','1')
          REQUEST.set('changetrans','0')
      elif buttonname == "add":
          self.add_transition(REQUEST)
          REQUEST.set('newtrans','0')
          REQUEST.set('changetrans','0')
      elif buttonname == "change":
          REQUEST.set('newtrans','0')
          REQUEST.set('changetrans','1')
      elif buttonname == "save":
          self.save_transition(REQUEST)
          REQUEST.set('newtrans','0')
          REQUEST.set('changetrans','0')    
      elif buttonname == "delete":
          self.delete_transition(REQUEST)
          REQUEST.set('newtrans','0')
          REQUEST.set('changetrans','0')      
      else:
          REQUEST.set('newtrans','0')
          REQUEST.set('changetrans','0')
          
      
      
  def get_transmode(self,REQUEST):
      # Belirtilen bir ge��n de�ri al��
      newtrans    =  REQUEST.get('newtrans','')
      changetrans =  REQUEST.get('changetrans','')
      if newtrans == "1":
          return "new"
      elif changetrans == "1":
          return "change"
      else:
          return "print"

  def gettable_transitions(self,REQUEST):
      # Tablo i�n kullan�acal ge��eri d�enler
      parentid = REQUEST.get('parentid','')
      if parentid == '':
          return []
      else:
          return self.qref.getChilds(parentid,'table_trans')
      
  def get_transition(self,REQUEST):
      # Belirtilen bir ge��n de�ri al��
      transmode = self.get_transmode(REQUEST)
      buttonvalue =  REQUEST.get('buttonvalue','')
      if transmode == 'new':
          return {'state':"",'destination':"",
                  'title':"",'description':"",'priority':"",'deadtime':""}
      
      elif transmode == 'change':
          if buttonvalue != "":
              return self.qref.getAttributes(buttonvalue)
      
      else:
          return {}
      
          
      
  def add_transition(self,REQUEST):
      # Tabloya ge��ekler
      temp_dic = {}
      temp_dic['tagname']     = 'table_trans'
      temp_dic['state']     = REQUEST.get('state','')
      temp_dic['destination'] = REQUEST.get('destination','')
      temp_dic['title']       = REQUEST.get('title','')
      temp_dic['description'] = REQUEST.get('description','')
      temp_dic['priority']    = REQUEST.get('priority','0')
      temp_dic['deadtime']    = REQUEST.get('deadtime','0')
      transobj = get_object(self.qref,self.qdata,temp_dic)
      parentid = REQUEST.get('parentid','')
      if parentid != "":
          transobj.insert(parentid)  
      
      
  def save_transition(self,REQUEST):
      # Tablodaki gecisi gunceller
      temp_dic = {}
      temp_dic['tagname']     = 'table_trans'
      temp_dic['tagid']       = REQUEST.get('transid','')
      temp_dic['state']       = REQUEST.get('state','')
      temp_dic['destination'] = REQUEST.get('destination','')
      temp_dic['title']       = REQUEST.get('title','')
      temp_dic['description'] = REQUEST.get('description','')
      temp_dic['priority']    = REQUEST.get('priority','0')
      temp_dic['deadtime']    = REQUEST.get('deadtime','0')
      transobj = get_object(self.qref,self.qdata,temp_dic)
      transobj.update(temp_dic)  
  
  def delete_transition(self,REQUEST):
      # Tablodaki ge�� siler
      temp_dic = {}
      temp_dic['tagname']  = 'table_trans'
      transIds             = REQUEST.get('transid','')
      for transId in transIds:
          temp_dic['tagid']  = transId
          transobj = get_object(self.qref,self.qdata,temp_dic)
          transobj.delete()  
    
########################################################################
########################################################################
########################################################################    
    
    
  def getText(self,text):
      # Message Catalog'dan aranilan text'in secili dil karsiligi alinir
      text = self.aq_parent.kzTranslateTextScript(text)
      if type(text) is UnicodeType:
          text = text.encode('utf-8')
      return text


########################################################################
###################### CREATE DATABASE FROM REFERENCE ##################
########################################################################



  def createDBfromRef(self,parentID="",nodeID="g0",container=None):
    # Referans veritaban�sql veritaban�t� 
    node      = self.qref.getAttributes(nodeID)
    obj       = get_object(self.qref,self.qdata,node,container)
    obj.tagid = nodeID
    if obj.create_db(parentID):
        print nodeID
        if obj.attr["tagname"] in ["group",'table']:
            childs = self.qref.getChilds(nodeID)
            for child in childs:
                if child["tagname"] == "table" and child["type"] == "m":
                    self.createDBfromRef(nodeID,child["tagid"],container)
            for child in childs:
                if child["tagname"] != "table" or child["type"] != "m":
                    self.createDBfromRef(nodeID,child["tagid"],container)        
    return "OK"


  def carryReference(self,qref_source,qref_dest,node_source="g0",parent_dest=""):
      # Referans veritaban�ska bir yere tasir
      import sys
      
      def getObject(container,id,meta_name=None):
	# Belirtilen container'in altindaki belirtilen obje dondurulur
	if hasattr(container,id):
	    obj = getattr(container,id)
	    if obj.aq_parent == container:
		return obj
	    else:
		return None
	  
      node      = qref_source.getAttributes(node_source)
      if not node["tagname"] in ["sqlselect",'sqlupdate']:
          obj   = get_object(qref_dest,None,node)
          try:
              node_dest = obj.insert_ref(parent_dest)
              print "%s" % (node['tagid'])
          except:
              print node
              raise "%s %s %s" % (sys.exc_info,sys.exc_type,sys.exc_value)
      # Tablonun tasinmaz objelerini ta�
      if node["tagname"] == "table":
          objs = ['oninsert','onupdate','ondelete']
      # Alan� ta�nmaz objelerini tasi
      elif node["tagname"] == "field":
          objs = ['oninsert']
      else:
          objs = []
          
      if len(objs):
          cont_source = qref_source.getContainer(node_source)
          cont_dest   = qref_dest.getContainer(node_dest)
          # Dosyalar yerlerinde mi diye kontrol ediliyor
          objsexists = []
          for o in objs:
              if getObject(cont_source,o) != None:
                  objsexists.append(o)
          # Dosyalar tas��or
          cp     = cont_source.manage_cutObjects(objsexists)
          result = cont_dest.manage_pasteObjects(cp)          
      
      for child in qref_source.getChilds(node_source):
          self.carryReference(qref_source,qref_dest,child['tagid'],node_dest)
      
      return "OK"
      
    
    
  def refreshObjects(self,nodeID="g0"):  
    # Referanstaki objeleri g�celler
    node      = self.qref.getAttributes(nodeID)
    obj       = get_object(self.qref,self.qdata,node)
    if obj is not None:
        try:
            obj.update(obj.attr)
            for child in self.qref.getChilds(nodeID):
                self.refreshObjects(child["tagid"])
        except:
            pass
    
    return "OK" 

  def addsequence(self,nodeID="g0"):
      node = self.qref.getAttributes(nodeID)
      if node["tagname"] == "table":
          obj = get_object(self.qref,self.qdata,node)
          obj.createsequence()
      if node["tagname"] in ["group",'table']:
          childs = self.qref.getChilds(nodeID)
          for child in childs:
              self.addsequence(child["tagid"])
      return "OK"  
  
  
  def setsequences(self,nodeID="g0"):
      node = self.qref.getAttributes(nodeID)
      if node["tagname"] == "table":
          obj = get_object(self.qref,self.qdata,node)
          obj.setsequence()
      if node["tagname"] in ["group",'table']:
          childs = self.qref.getChilds(nodeID)
          for child in childs:
              self.setsequences(child["tagid"])
      return "OK"  
  
  
########################################################################  
########################################################################  
######################## ARCHIEVE  #####################################  
########################################################################  

  def checkDBForArchive(self):
      "check for all records for archive"
      for group in self.qref.getAllGroups("g0",restriction=["isleaf","==","T"]):
          self.checkGroupForArchive(group["tagid"])
      
                  
  def checkGroupForArchive(self,groupid):
      "check for all records for archive"
      DayOfWeek_list = ['','Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday','Sunday']
      files_fields_list = {'id'     :{"name":"id","type":"varchar(250)","primary":""},
                           'groupid':{"name":"groupid","type":"varchar(250)"},
                           'recno'  :{"name":"recno","type":"integer"},
                           'name'   :{"name":"name","type":"varchar(250)"},
                           'type'   :{"name":"type","type":"varchar(250)"},
                           'arch'   :{"name":"arch","type":"varchar(1)"},
                           'auth'   :{"name":"auth","type":"integer"},
                           'content':{"name":"content","type":""}
                           }
      
      
      
      
      def get_date(life=365):
          date = str( DateTime()-int(life) )
          date = date[:date.find('.')]
          date = date.replace(':','/')
          return date
      
      def createFileTable():
          sqlobj = self.qdata.getsqlobject()
          table_info = sqlobj.get_table_info("files","qarchives")
          if table_info is None:
              blobtype = sqlobj.getblobtype()
              files_fields_list["content"]["type"] = blobtype
              fields = []
              for files_field in files_fields_list:
                  fields.append( files_fields_list[files_field] )
              sqlobj.create_table("files",fields)
              self.qdata.results_qarchives(sqlobj)

      def copyFiles(recno):
          rec = self.qdata.sqlselectrecords("files",['*'],cond=(('groupid',groupid),'and',('arch','1')),auth=0)
          sqlobj1 = self.qdata.getsqlobject()
          sqlobj2 = self.qdata.getsqlobject()
          for r in rec:
              dic = {'id'     : r.get("id",str(DateTime())),
                     'groupid': r.get("groupid",""),
                     'recno'  : r.get("recno",-1),
                     'name'   : r.get("name",""),
                     'type'   : r.get("type",""),
                     'arch'   : r.get("arch","1"),
                     'auth'   : r.get("auth",-1),
                     'content': sqlobj2.setBinary(r["content"])
                     }
              sqlobj1.selectrecords("files",["id"],cond=("id",r["id"]),auth=0)
              if len(self.qdata.results_qarchives(sqlobj1)):
                  sqlobj2.updaterecords("files",dic,cond=("id",r["id"]),auth=0)
              else:
                  sqlobj2.insertrecords("files",dic)
          self.qdata.results_qarchives(sqlobj2)                  
                    
      
      table_hierarchy = ()
      def createTableHierarchy():
          # table_hierarchy = (attr,fields,sublevel_tables,parent)
          master = self.qref.getChilds(groupid,tagName='table',restriction=["type","==","m"]) 
          if not len(master) or len(master)>1:
              return "no_master"
          else:
              fields = self.qref.getChilds(master[0]["tagid"],tagName='field')
              master[0]["parentid"] = groupid
              table_hierarchy = (master[0],fields,[],None)
              sub_table = self.qref.getChilds(master[0]["tagid"],tagName='table')
              if len(sub_table):
                  fields = self.qref.getChilds(sub_table[0]["tagid"],tagName='field')
                  sub_table[0]["parentid"] = master[0]["tagid"]
                  table_hierarchy[2].append( (sub_table[0],fields,[],table_hierarchy) )
              
          non_masters = self.qref.getChilds(groupid,tagName='table',restriction=["type","!=","m"]) 
          for non_master in non_masters:
              fields = self.qref.getChilds(non_master["tagid"],tagName='field')
              non_master["parentid"] = groupid
              temp = (non_master,fields,[],table_hierarchy)
              sub_table = self.qref.getChilds(non_master["tagid"],tagName='table')
              if len(sub_table):
                  fields = self.qref.getChilds(sub_table[0]["tagid"],tagName='field')
                  sub_table[0]["parentid"] = non_master["tagid"]
                  temp[2].append( (sub_table[0],fields,[],temp) )
              table_hierarchy[2].append(temp)
          return table_hierarchy    
              
              
      def createTableSchemma():
          # Master Level
          syncTable(table_hierarchy)
          # Level 1
          for table in table_hierarchy[2]:
              syncTable(table)
              for subtable in table[2]:
                  syncTable(subtable)

              
      def syncTable(table):
          sqlobj = self.qdata.getsqlobject()
          table_info = sqlobj.get_table_info(table[0]["tagid"],"qarchives")
          if table_info is None:
              createTable(table)
          else:
              fields_arch = table_info.keys()
              for field in table[1]:
                  if not ((field["tagid"] in fields_arch) or (field["tagid"]+"_0" in fields_arch)):
                      fieldObj = get_object(self.qref,self.qdata,field)
                      fieldObj.insert_data(table[0]["tagid"],"results_qarchives")    
           
      def createTable(table):
          #qdata archive ile degisecek
          tableObj = get_object(self.qref,self.qdata,table[0])
          tableObj.insert_data(table[0]["parentid"],"results_qarchives")
          for field in table[1]:
              if field['type'] == 'd':
                  field['type'] = 'da'
              fieldObj = get_object(self.qref,self.qdata,field)
              fieldObj.insert_data(table[0]["tagid"],"results_qarchives")
              
              
      def selectrecords(table):
          life = table[0]["life"]
          try: cond = ('date','<',get_date(int(life)-1000))
          except: cond = ('date','<',get_date(365))
          return self.qdata.sqlselectrecords(table[0]["tagid"],['recno'],cond=cond,auth=0)
      
      
      def checkforeign(table,masterrecno):
          table_recno = []
          parent = table[3]
          masterrecno=[masterrecno]
          while parent!=None and masterrecno!=[None]:
              cond=()
              for mrecno in masterrecno:
                  if len(cond): cond = (cond,"or",("recno",mrecno))
                  else: cond = ("recno",mrecno)

              if parent[0]["type"] == "m":
                  rec = self.qdata.sqlselectrecords(parent[0]["tagid"],["recno"],cond=cond,auth=0)
                  list=[]
                  masterrecno = [None]
                  for r in rec:
                      recno = r["recno"]
                      list.append( {"recno":recno} )
                  table_recno.insert(0,(parent[0]["tagid"],list))
                      
              elif parent[0]["type"] == "d":  
                  rec = self.qdata.sqlselectrecords(parent[0]["tagid"]+"r",["recno","master"],cond=cond,auth=0)
                  list = [{"recno":masterrecno}]
                  table_recno.insert(0,(parent[0]["tagid"],list))
                  masterrecno = []
                  temp_list = []
                  for r in rec:
                      temp_list.append( {"master":r["master"],"recno":r["recno"]} )
                      masterrecno.append(r["master"])
                  table_recno.append(0,(parent[0]["tagid"]+"r",list))
                  
              else:
                  rec = self.qdata.sqlselectrecords(parent[0]["tagid"],["recno","masterrecno"],cond=cond,auth=0)
                  masterrecno = [rec[0]["masterrecno"]]
                  list = [{"recno":recno,"masterrecno":masterrecno[0]}]
                  table_recno.insert(0,(parent,list))
                  
              parent = parent[3]
              
          # Control archieve for foreign keys
          sqlobj = self.qdata.getsqlobject()
          for t_r in table_recno:
              tableid  = t_r[0]
              rec_list = t_r[1]
              for rec in rec_list:
                  for key in rec.keys():
                      if len(cond): cond = (cond,"or",(key,rec[key]))
                      else: cond = (key,rec[key])
                  sqlobj.selectrecords(tableid,["recno"],cond=cond,auth=0)
                  if not len( self.qdata.results_qarchives(sqlobj) ):
                      sqlobj.insertrecords(tableid,rec)
                      self.qdata.results_qarchives(sqlobj)
              
           
      
      def copyrecord(table,record):
          checkforeign(table,record.get("masterrecno",-1))
          record = modify_record(table[1],record)
          recno  = record["recno"]
          sqlobj = self.qdata.getsqlobject()
          sqlobj.selectrecords(table[0]["tagid"],["recno"],cond=("recno",recno))
          rec = self.qdata.results_qarchives(sqlobj)
          if len(rec):
              sqlobj.updaterecords(table[0]["tagid"],dic=record,cond=("recno",recno),auth=0)
          else:
              sqlobj.insertrecords(table[0]["tagid"],dic=record)
          self.qdata.results_qarchives(sqlobj) 
          

      def setquarter(month):
          "0:year,1:quarter,2:month,3:day,4:dayofweek"
          if month<4: return 1
          elif month<7: return 2
          elif month<10: return 3
          else: return 4

      def modify_record(fields,record):
          dict = {}
          from kzTables import essential_fields
          templist=["","_0","_1","_2","_3","_4","_5"]
          for field in fields:
              if field["type"] in ["d","da"]:
                  from DateTime import DateTime
                  value = record.get(field["tagid"]+"_0","")
                  try:
                      date      = DateTime(value)
                      year      = date.year()
                      month     = date.month()
                      quarter   = setquarter(month)
                      day       = date.day()
                      DayOfWeek = DayOfWeek_list.index(date.Day())
                  except:
                      year=month=quarter=day=DayOfWeek=0
                      
                  dict[field["tagid"]+"_0"] = year
                  dict[field["tagid"]+"_1"] = quarter
                  dict[field["tagid"]+"_2"] = month
                  dict[field["tagid"]+"_3"] = day
                  dict[field["tagid"]+"_4"] = DayOfWeek
              else:
                  for temp in templist:
                      if record.has_key(field["tagid"]+temp):
                          dict[field["tagid"]+temp] = record[field["tagid"]+temp]
                      
          for essential_field in essential_fields:
              if record.has_key(essential_field):
                  dict[essential_field] = record[essential_field]

          return dict
                  
                  
                  
          
      def master2archive(table,recno):
          "copy table's and related table's records to archieve"
          #begin_transaction
          #end_transaction
          cond = ('recno','=',recno)
          for record in self.qdata.sqlselectrecords(table[0]["tagid"],["*"],cond=cond,auth=0):
              copyrecord(table,record)
              for sub_table in table[2]:
                  sub_records = self.qdata.sqlselectrecords(sub_table[0]["tagid"],['recno'],cond=('masterrecno','=',recno))
                  for sub_record in sub_records:
                      sub_recno = sub_record['recno']
                      master2archive(sub_table,sub_recno)

                            

      #---------------start--------------------------------------------#
      createFileTable()
      table_hierarchy = createTableHierarchy()
      if table_hierarchy == "no_master":
          return
      createTableSchemma()
      # Master Level
      records = selectrecords(table_hierarchy)
      for rec in records:
          master2archive(table_hierarchy,rec["recno"])
          copyFiles(rec["recno"])
          self.qdata.sqldeleterecords(table_hierarchy[0]["tagid"],cond=("recno",rec["recno"]),auth=0)
          self.qdata.sqldeleterecords("files",cond=(("groupid",groupid),"and",("recno",rec["recno"])),auth=0)
      # Alt d�eyler i�n
      for sub_table in table_hierarchy[2]:
          subrecords = selectrecords(sub_table)
          for rec in subrecords:
              master2archive(sub_table,rec["recno"])
              self.qdata.sqldeleterecords(sub_table[0]["tagid"],cond=("recno",rec["recno"]),auth=0)
          for subsub_table in sub_table[2]:
              subsubrecords = selectrecords(subsub_table)
              for rec in subsubrecords:
                  master2archive(subsub_table,rec["recno"])
                  self.qdata.sqldeleterecords(subsub_table[0]["tagid"],cond=("recno",rec["recno"]),auth=0)
      
      
      
      return 
  
 
#########################################################################        
###################    GET OBJECT   #####################################
#########################################################################

def get_object(qref,qdata,dic,container=None):
    "Belirtilen �liklere g�nesneyi d�r�  "
    tagname = dic["tagname"]

    if dic.has_key("tagid"):
        tagid   = dic["tagid"]
    else:
        tagid   = ""
        
    temp = None
    
    if tagname == "group":
        temp = kzGroup(qref,qdata,tagid,dic)
                       
    elif tagname == "table":
        if dic["type"] == "m":
            temp = kzTable_m(container,qref,qdata,tagid,dic)
        elif dic["type"] == "1":
            temp = kzTable_1(container,qref,qdata,tagid,dic)
        elif dic["type"] == "n":
            temp = kzTable_n(container,qref,qdata,tagid,dic)
        elif dic["type"] == "d":
            temp = kzTable_nn(container,qref,qdata,tagid,dic)    
        elif dic["type"] == "s":
            temp = kzTable_s(container,qref,qdata,tagid,dic)   
        else:
            temp = kzTable(container,qref,qdata,tagid,dic)   
            
    elif tagname == "field":
        temp = kzField(container,qref,qdata,tagid,dic)

    elif tagname == "select":
        temp = kzSelection(qref,tagid,dic)
        
    elif tagname == "param":
        temp = kzParameter(qref,tagid,dic)  
        
    elif tagname in ["dynamicprops","script","oninsert","beforeinsert","onupdate","beforeupdate","ondelete","beforedelete","ontime"]:
        temp = kzScript(qref,tagid,dic)  
        
    elif tagname in ["javascript","onclick","onchange","onfocus","onblur","onmouseover","onmouseout","onkeypress"]:
        temp = kzJavascript(qref,tagid,dic)      
        
    elif tagname == "table_trans":
        temp = kzTableTransitions(qref,tagid,dic)      
        
    elif tagname == "templates":
        temp = kzTemplates(qref,tagid,dic)    
        
    elif tagname == "form":
        temp = kzForms(qref,tagid,dic)       
        
    return temp 

                
    
########################################################################
########################################################################
########################################################################
########################################################################
    

# Administrative pages

def manage_addProduct(self, id, title, qref, qdata,RESPONSE=None):
  " kzDBDesign product'�r "
  if (id=="") or (qref=="") or (qdata==""):
    RESPONSE.redirect('addProductForm')
      
  else:
    temp       = kzDBDesign(id)
    
    setattr(temp,'qdata',getattr(self,qdata))
    temp.qdataID = qdata

    setattr(temp,'qref',getattr(self,qref))
    temp.qrefID = qref             
            
    temp.title     = title
    
    self._setObject(id, temp)
    
    if RESPONSE != None:
      RESPONSE.redirect('manage_workspace')
    
    
  
 
