# -*- coding: utf-8 -*- 
# -*- coding: utf-8 -*- 
from kzObjects import *
import copy

try: from Products.Sequences.SequenceManager import SequenceManager
except: pass

table_attr = ["title","order","type","life","description","enable","style","mastersummary","formbuttons"]

essential_fields = ["recno","masterrecno","date","lastuser","caption","authority","owner_name","state"]

class kzTable:
  
  def __init__(self,container,qref,qdata,tagid,attr):
    "Tablo nesnesi olusturur"
    
    self.tagname    = "table"
    self.tagid      = tagid
    self.qref       = qref
    self.qdata      = qdata
    self.container  = container
    self.attr       = self.control_attr(attr) 
    self.sql        = sql(qdata) 
    self.menu_count = 4
    
  def getAttributes(self):
    dic = {"tagid" : self.tagid}
    dic.update(self.attr)
    return dic
  
  def isEnabled(self):
    return self.attr.get("enable","0") in [1,"1"]
    
  def create_db(self,parentID):
    # Veritabani yaratilirken yapilacak islem
    self.insert_data(parentID)
    return 1
  
  def getsequence(self):
    container = self.qref.getContainer(self.tagid)
    if 'sequence' in container.objectIds():
      return getattr(container,'sequence')
    else:
      return None

      
  def createsequence(self):
    # Tablo i�n unique bir sequencer yarat�
    container = self.qref.getContainer(self.tagid)
    if self.getsequence() == None:
      container = self.qref.getContainer(self.tagid)
      container._setObject('sequence', SequenceManager("sequence"))
    else:
      container._delObject('sequence')
      container._setObject('sequence', SequenceManager("sequence"))

      
  def setsequence(self):
    sqlobj = self.qdata.getsqlobject()
    sqlobj.query("select max(recno) as recno from %s" % self.tagid)
    rec = self.qdata.results_qdata(sqlobj)
    if len(rec):
      from Missing import Missing
      if rec[0]["recno"] == None:
        recno = 0
      elif type(rec[0]["recno"]) == type(Missing()):
        recno = 0  
      else:
        recno = int(rec[0]["recno"]) 
    else:
      recno = 0
    sequence = self.getsequence()
    if not sequence:
      self.createsequence()
      sequence = self.getsequence()
    self.resetsequence(recno)
    
  def resetsequence(self,value=0):
    sequence = self.getsequence()
    if sequence:
      while sequence.getNextValue(self.tagid) < value:
        pass
    
  def insert(self,parentID):
    # Yeni bir tablo ekler
    # Referansa ekle
    self.tagid = self.insert_ref(parentID)
    self.insert_data(parentID)
    return self.tagid
    
    
  def insert_ref(self,parentID):
    # Referansa yeni bir tablo ekler
    self.tagid = self.qref.insertNode(parentID,"table",self.attr)
    self.createsequence()
    return self.tagid

  
  def insert_data(self,parentID):
    pass
    
  
  def update(self,attr):
    # Tablonun referans �liklerini de�irir
    attr = control_attr(attr)
    attr["type"]    = self.attr["type"]
    attr["tagname"] = self.tagname
    # Referans�elle
    self.qref.updateNode(self.tagid,attr)
    for key in attr.keys():
      self.attr[key] = attr[key]
   
  def delete(self):
    # Tabloyu veritaban�siler
    # Referanstan sil  
    self.delete_ref()
    
  def delete_ref(self):
    # Referanstan sil
    self.qref.delete(self.tagid)  
    
  def delete_data(self):
    # Referanstan sil
    self.sql.drop_table(self.tagid)
  
  def copy(self,parentID,references=1):
    # Tabloyu kopyalar
    new_tagid = self.qref.copy(self.tagid,parentID)
    self.copy_data(new_tagid)
    if references:
      temp = kzTable(self.qref,self.qdata,new_tagid,self.attr)
      temp.delete_field_references()
    return new_tagid
  
  def copy_data(self,new_tagid,truncate=0):
    # Tabloyu yeni isimde kopyalar
    sqlobj = self.qdata.getsqlobject()
    sqlobj.query("create table %s as select * from %s" % (new_tagid,self.tagid))
    #Tablonun alanlarinin adini degistir
    fields = sqlobj.get_table_info(self.tagid)
    for field in fields.keys():
      field_id     = fields[field]["name"]
      if not field_id in essential_fields:
        new_field_id = new_tagid + field_id[len(self.tagid):]
        sqlobj.rename_field(new_tagid,field_id,new_field_id)

    self.qdata.results_qdata(sqlobj)

    from kzDBDesign import get_object
    # Tablonun primary key'lerini olustur
    attr = copy.deepcopy(self.attr)
    attr['tagid'] = new_tagid
    temp = get_object(self.qref,self.qdata,attr)
    temp.modify_copied()

    if truncate:
      temp.truncate()
      
    
  def modify_copied(self):
    # Kopyalanan tablonun primary ve foreign keylerini olusturur
    sqlobj = self.qdata.getsqlobject()
    sqlobj.add_constraint(self.tagid,'recno','primary')
    self.qdata.results_qdata(sqlobj)
    
    
  def find_mastertable(self):
    # Tablonun ana tablosu bulunur
    parentid = get_parent(self.tagid)
    parent = self.qref.getAttributes(parentid)
    while parent["tagname"] != "group":
      parentid = get_parent(parentid)
      parent   = self.qref.getAttributes(parentid)
    mastertable = self.qref.getChilds(parentid,'table',restriction=['type','==','m'])  
    if len(mastertable):
      return mastertable[0]['tagid']
    else:
      return None
    
    
  def truncate(self):
    # Tablonun i�i� bo�alt
    sqlobj = self.qdata.getsqlobject()
    sqlobj.truncate_table(self.tagid)
    self.qdata.results_qdata(sqlobj)
    self.resetsequence()
      
    
  def delete_field_references(self):
    # Tablonun alanlar�nup referanslar�nir
    dic = {}
    dic["reference"] = ""
    dic["essential"] = "0"
    dic["source"]    = "default"
    for child in self.qref.getChilds(self.tagid):
      if child["tagname"] == "field":
        self.qref.updateNode(child["tagid"],dic)
      elif child["tagname"] == "table":
        for child2 in self.qref.getChilds(child["tagid"]):
          self.qref.updateNode(child2["tagid"],dic)
        
  def control_attr(self,attr):  
    # Tablonun �liklerini kontrol et
    attr = copy.deepcopy(attr)
    keys = attr.keys()
    dic  = {}
    for item in cmp_list(keys,table_attr): 
      if item == "formbuttons":
        if self.tagid:
          buttons = self.container.kzDBObjectProperties.getKzTableFormButtons(self.tagid) 
          value = ",".join(buttons)
        else:
          value = ''
      else:
        value = ''
      dic[item] = attr[item]= value
    if len(dic): 
      self.qref.updateNode(self.tagid,dic)  
    attr["tagname"] = self.tagname
    attr["deleted"] = "0"
    attr["enable"] = str( str(attr["enable"]) and str(attr["enable"]) or 1 )
    return control_attr(attr)
  
  
  
class kzTable_m(kzTable):
  
  def insert_data(self,parentID="",results="results_qdata"):
    # M tipi tabloyu sisteme ekler
    fields = [{'name':'recno'     ,'type':'integer','primary':''},
              {'name':'lastuser'  ,'type':'varchar(250)'},
              {'name':'date'      ,'type':'varchar(250)'},
              {'name':'authority' ,'type':'integer'},
              {'name':'owner_name','type':'varchar(250)'},
              {'name':'state'     ,'type':'varchar(250)'}]

    sqlobj = self.qdata.getsqlobject()
    sqlobj.create_table(self.tagid,fields)
    getattr(self.qdata,results)(sqlobj)
    

class kzTable_1(kzTable):

  def insert_data(self,parentID="",results="results_qdata"):
    # 1 tipi tabloyu sisteme ekler
    mastertable = self.qref.getChilds(parentID,'table',restriction=['type','==','m'])
    if len(mastertable): 
      masterid = mastertable[0]['tagid']
    else:
      masterid = ''
    
    fields = [{'name':'recno'      ,'type':'integer','primary':''},
              {'name':'masterrecno','type':'integer','foreign':{'tablename':masterid,'fieldname':'recno'}},
              {'name':'lastuser'   ,'type':'varchar(250)'},
              {'name':'date'       ,'type':'varchar(250)'},
              {'name':'authority'  ,'type':'integer'},
              {'name':'owner_name' ,'type':'varchar(250)'},
              {'name':'state'      ,'type':'varchar(250)'}]
    sqlobj = self.qdata.getsqlobject()
    sqlobj.create_table(self.tagid,fields)
    getattr(self.qdata,results)(sqlobj)
    
  def modify_copied(self):
    # Kopyalanan tablonun primary ve foreign keylerini olusturur
    kzTable.modify_copied(self)
    masterid = self.find_mastertable()
    if masterid is not None:
      sqlobj = self.qdata.getsqlobject()
      sqlobj.add_constraint(self.tagid,'masterrecno','foreign',{'tablename':masterid,'fieldname':'recno'})
      self.qdata.results_qdata(sqlobj)
      
    
class kzTable_n(kzTable):

  def insert_data(self,parentID="",results="results_qdata"):
    # N tipi tabloyu sisteme ekler
    mastertable = self.qref.getChilds(parentID,'table',restriction=['type','==','m'])
    if len(mastertable): 
      masterid = mastertable[0]['tagid']
    else:
      masterid = ''
    
    fields = [{'name':'recno'      ,'type':'integer','primary':''},
              {'name':'masterrecno','type':'integer','foreign':{'tablename':masterid,'fieldname':'recno'}},
              {'name':'lastuser'   ,'type':'varchar(250)'},
              {'name':'date'       ,'type':'varchar(250)'},
              {'name':'authority'  ,'type':'integer'},
              {'name':'owner_name' ,'type':'varchar(250)'},
              {'name':'state'      ,'type':'varchar(250)'}]
    sqlobj = self.qdata.getsqlobject()
    sqlobj.create_table(self.tagid,fields)
    getattr(self.qdata,results)(sqlobj)
    
  def modify_copied(self):
    # Kopyalanan tablonun primary ve foreign keylerini olusturur
    kzTable.modify_copied(self)
    masterid = self.find_mastertable()
    if masterid is not None:
      sqlobj = self.qdata.getsqlobject()
      sqlobj.add_constraint(self.tagid,'masterrecno','foreign',{'tablename':masterid,'fieldname':'recno'})
      self.qdata.results_qdata(sqlobj) 
    
  
class kzTable_nn(kzTable):

  def insert_data(self,parentID="",results="results_qdata"):
    # N tipi tabloyu sisteme ekler
    mastertable = self.qref.getChilds(parentID,'table',restriction=['type','==','m'])
    if len(mastertable): 
      masterid = mastertable[0]['tagid']
    else:
      masterid = ''
      
    fields = [{'name':'recno'      ,'type':'integer','primary':''},
              {'name':'masterrecno','type':'integer'},
              {'name':'lastuser'   ,'type':'varchar(250)'},
              {'name':'date'       ,'type':'varchar(250)'},
              {'name':'authority'  ,'type':'integer'},
              {'name':'caption'    ,'type':'varchar(250)'},
              {'name':'owner_name' ,'type':'varchar(250)'},
              {'name':'state'      ,'type':'varchar(250)'}]  

    sqlobj = self.qdata.getsqlobject()
    sqlobj.create_table(self.tagid,fields)
    
    
    fields = [{'name':'recno'  ,'type':'integer','foreign':{'tablename':self.tagid,'fieldname':'recno'}},
              {'name':'master' ,'type':'integer','foreign':{'tablename':masterid,'fieldname':'recno'}}]  
    sqlobj.create_table(self.tagid+"r",fields)
    getattr(self.qdata,results)(sqlobj)
    
    
    
  def modify_copied(self):
    # Kopyalanan tablonun primary ve foreign keylerini olusturur
    kzTable.modify_copied(self)
    masterid = self.find_mastertable()
    if masterid is not None:
      sqlobj = self.qdata.getsqlobject()
      sqlobj.add_constraint(self.tagid+'r','recno' ,'foreign',{'tablename':self.tagid,'fieldname':'recno'})
      sqlobj.add_constraint(self.tagid+'r','master','foreign',{'tablename':masterid,'fieldname':'recno'})
      self.qdata.results_qdata(sqlobj)  
  
    
  def copy_data(self,new_tagid,truncate=0):
    # Tabloyu yeni isimde kopyalar
    sqlobj = self.qdata.getsqlobject()
    sqlobj.query("create table %sr as select * from %sr" % (new_tagid,self.tagid))
    self.qdata.results_qdata(sqlobj)
    kzTable.copy_data(self,new_tagid,truncate)
    
    
  def truncate(self):
    # Tablonun i�i� bo�alt
    sqlobj = self.qdata.getsqlobject()
    sqlobj.truncate_table(self.tagid)
    sqlobj.truncate_table(self.tagid+"r")
    self.qdata.results_qdata(sqlobj)
    self.resetsequence()
    
class kzTable_s(kzTable):

  def insert_data(self,parentID="",results="results_qdata"):
    # S tipi tabloyu sisteme ekler
    # Veritaban�le
    masterid = get_parent(self.tagid)

    mastertable = self.qref.getAttributes(masterid)
    if mastertable['tagname'] == 'group':
      self.qref.updateNode(self.tagid,{'type':'n'})
      self.attr["type"] = "n"
      masterid = ""

    fields = [{'name':'recno'      ,'type':'integer','primary':''},
              {'name':'masterrecno','type':'integer','foreign':{'tablename':masterid,'fieldname':'recno'}},
              {'name':'lastuser'   ,'type':'varchar(250)'},
              {'name':'date'       ,'type':'varchar(250)'},
              {'name':'authority'  ,'type':'integer'},
              {'name':'owner_name' ,'type':'varchar(250)'},
              {'name':'state'      ,'type':'varchar(250)'}]
    sqlobj = self.qdata.getsqlobject()
    sqlobj.create_table(self.tagid,fields)
    getattr(self.qdata,results)(sqlobj)
    
    
  def modify_copied(self):
    # Kopyalanan tablonun primary ve foreign keylerini olusturur
    kzTable.modify_copied(self)
    masterid = get_parent(self.tagid)
    mastertable = self.qref.getAttributes(masterid)
    if mastertable['tagname'] == 'table':
      sqlobj = self.qdata.getsqlobject()
      sqlobj.add_constraint(self.tagid,'masterrecno','foreign',{'tablename':masterid,'fieldname':'recno'})
      self.qdata.results_qdata(sqlobj)   
    
  
    
