# -*- coding: utf-8 -*- 
# -*- coding: utf-8 -*- 
from Products.CMFCore.utils             import UniqueObject
from OFS.SimpleItem                     import SimpleItem
from Globals                            import InitializeClass
from OFS.ObjectManager                  import ObjectManager
from Products.PageTemplates             import PageTemplateFile
from Products.Kuzgun.kzXMLSentence      import kzXMLSentence
from Products.Kuzgun.kzDate.kzDate      import kzDate
from Products.Kuzgun.kzString           import kzString
from Products.Kuzgun.kzSort             import kzSort
import copy
import string
from Products.ZSQLMethods.SQL import SQL
from DateTime import DateTime

from array import array

ARRAY_TYPE = array

class kzFileManager(SimpleItem):
  """File Manager"""
  id='kzFileTool'
  meta_type='kzFileManager'
  plone_tool=1
  LFSName="Local_FS"
  SQLName="kzAuthSQL"
  SQLTable="files" # id,groupid,recno,name,type,arch,auth,content
  
########################################################################
##################### FILE PROCESS #####################################
########################################################################

  def fileProcess(self,request):

    if request.has_key("reset"):
      # Veritabani erisimi
      self.qattach = getattr(self.aq_parent,self.SQLName)
      # Acilis ayarlarini yap
      request.SESSION["FileManager"] = {}
      SESSION = request.SESSION["FileManager"]
      groupid = request.get("groupid","")
      filter  = request.get("filter","")
      recno   = request.get("recno",0)
      mode    = int(request.get("mode",0))
      if len(groupid) and len(recno):
        SESSION["GroupID"] = groupid
        try: 
          float(recno)
          if recno>0:
            SESSION["Recno"] = recno
          else:
            title    = "Recno Error"
            subtitle = "Recno must be greater than 0"
            message  = "Contact to your Administrator"
            self.aq_parent.setError(title,subtitle,message)
            return 
        except:
          title    = "Request Error"
          subtitle = "Recno Type is not integer type"
          message  = "Contact to your Administrator"
          self.aq_parent.setError(title,subtitle,message)
          return 
      else:
        title    = "Request Error"
        subtitle = "No GroupID or Recno Parameter"
        message  = "Contact to your Administrator"
        self.aq_parent.setError(title,subtitle,message)
        return 
      
      SESSION["ViewMode"] = ("filetable","") # Mode , related file id
      SESSION["Filter"]   = filter

      if mode: 
        SESSION["Mode"] = 1
        SESSION["Formname"]  = request.get("formname","")
        SESSION["Fieldname"] = request.get("fieldname","")       
      else: 
        SESSION["Mode"] = 0
        
      # Kullanc yetkisini al  
      username = self.aq_parent.portal_membership.getAuthenticatedMember()
      SESSION["Userauth"] = self.kzAuthSQL.getAuth(str(username),groupid)
        
      
    elif not request.SESSION["FileManager"].has_key("ViewMode"):
      title    = "SESSION Warning"
      subtitle = "Your Session time is ended"
      message  = "Please restart he file manager"
      self.aq_parent.setError(title,subtitle,message)
       
    
    else:
      SESSION = request.SESSION["FileManager"]
      buttonname  = request.get("buttonname","")
      buttonvalue = request.get("buttonvalue","")
      
      if buttonname == "openrename":
        SESSION["ViewMode"] = ("rename","")
        
      elif buttonname == "rename":  
        fileids = request.get("fileid",[])
        filenames = request.get("filename",[])
        self.renamefiles(fileids,filenames)
        SESSION["ViewMode"] = ("filetable","")  
      
      elif buttonname == "delete":
        self.deletefiles(request.get("files",[]))
        SESSION["ViewMode"] = ("filetable","")  
        
      elif buttonname == "new_file":
        SESSION["ViewMode"] = ("new","")
        
      elif buttonname == "file_properties":
        SESSION["ViewMode"] = ("properties",buttonvalue)
        
      elif buttonname == "cancel_propmode":
        SESSION["ViewMode"] = ("filetable","")  
        
      elif buttonname == "add_file":
        self.insertfile(request)
        SESSION["ViewMode"] = ("filetable","")  
        
      elif buttonname == "change_fileproperties":
        self.updatefile(request)
        SESSION["ViewMode"] = ("filetable","")
        
      elif buttonname == "openfile":
        path = self.createTempFile(buttonvalue)
        request.set("ViewFile",path)
        

        
    
########################################################################        
################### PATH MANAGEMENT ####################################
########################################################################      
      
  def getTempPath(self):
    return self.aq_parent.temp_folder
  
  
########################################################################        
################### TEMP FILE MANAGEMENT ###############################
########################################################################

  def createTempFile(self,fileid):
    file     = self.getfilefromDB(fileid,content=1)
    if len(file):
      temppath = self.getTempPath()
      isImage  = str(file[0].get("type","")).find("image")
      objectids= temppath.objectIds()
      content  = file[0].get("content","")
      
      if type(content) is ARRAY_TYPE:
        content = content.tostring()
        
      if fileid in objectids:
        temppath._delObject(fileid)
      if isImage >= 0:
        temppath.manage_addProduct["OFSP"].manage_addImage(fileid,file=content)
      else:
        temppath.manage_addProduct["OFSP"].manage_addFile(fileid,file=content)
      return "%s/%s" % (temppath.absolute_url(),fileid)
      
      
  
  
########################################################################        
##################### DB MANAGEMENT ####################################
########################################################################

  def getAllfilesFromDB(self,groupid,recno,type="",auth=0):
    sqlobj = self.qattach.getsqlobject()
    cond = ("groupid",str(groupid))
    cond = (cond,"and",("recno",int(recno)))
    cond = (cond,"and",("type","like","%s%s%s" % ("%",type,"%")))
    cond = (cond,"and",("auth","<=",int(auth)))
    fields= ["id","name","type","octet_length(content) AS size","arch","auth"]
    sqlobj.selectrecords(self.SQLTable,fields,cond=cond,auth=0)
    return self.qattach.results_qattach(sqlobj)
  
  def getfilefromDB(self,fileid,content=0):
    if len(fileid):
      self.qattach = getattr(self.aq_parent,self.SQLName)
      sqlobj = self.qattach.getsqlobject()
      cond = ("id",str(fileid))
      fields= ["id","name","type","octet_length(content) AS size","arch","auth"]
      if content:
        fields.append("content")
      sqlobj.selectrecords(self.SQLTable,fields,cond=cond,auth=0)
      return self.qattach.results_qattach(sqlobj)
    else:
      return []
  
  
  def getID(self):
    dt=DateTime()
    return "%s-%s-%s-%s-%s-%s" % (dt.year(),dt.month(),dt.day(),dt.hour(),dt.minute(),str(dt.second()).replace(".",""))
    
  
  def insertfile(self,request):
    file = request.get("content",None)
    if file is not None:
      content  = file.read()
      mimetype = file.headers['Content-Type']
      # Kullanc dosya ismi girmi mi?
      name     = request.get("filename","")
      if name == "":
        if file.filename == "":
          name = "temp"
        else:          
          name = file.filename
      sqlobj = self.aq_parent.kzAuthSQL.getsqlobject()    
      # Eklemek iin dic olustur  
      dic = {"id":self.getID(),
             "groupid":request.SESSION["FileManager"]["GroupID"],
             "recno":int(request.SESSION["FileManager"]["Recno"]),
             "name":name,
             "type":mimetype,
             "arch":request.get("filearch",""),
             "auth":int(request.get("fileauth","0")),
             "content":sqlobj.setBinary(content) }
      
      sqlobj.insertrecords(self.SQLTable,dic)
      return self.qattach.results_qattach(sqlobj)
    
  def updatefile(self,request):
    fileid = request.get("fileid","")
    if fileid != "":
      file = request.get("content",None)
      if file is not None:
        content  = file.read()
        mimetype = file.headers['Content-Type']
        fname    = file.filename
        # Kullanc dosya ismi girmi mi?
        name     = request.get("filename","")
        if name == "":
          if fname == "":
            name = "temp"
          else:          
            name = fname
        # Eklemek iin dic olustur  
        dic = {"name":name,
               "arch":request.get("filearch",""),
               "auth":int(request.get("fileauth","0")),
               }
        sqlobj = self.aq_parent.kzAuthSQL.getsqlobject()
        if fname != "":
          dic["type"] = mimetype
          dic["content"] = sqlobj.setBinary(content)
        # Kst olustur
        cond = ("id",str(fileid))
        sqlobj.updaterecords(self.SQLTable,dic,cond,auth=0)
        return self.qattach.results_qattach(sqlobj)
      
  def deletefiles(self,fileids):
    cond=()
    for fileid in fileids:
      if len(cond):
        cond = (cond,"or",("id",str(fileid)))
      else:
        cond = ("id",str(fileid))
        
    if len(cond):    
      sqlobj = self.aq_parent.kzAuthSQL.getsqlobject()
      sqlobj.deleterecords(self.SQLTable,cond,auth=0)
      return self.qattach.results_qattach(sqlobj)
    
    
  def renamefiles(self,fileids,filenames):
    sqlobj = self.aq_parent.kzAuthSQL.getsqlobject()
    control = 0
    for i in range(len(fileids)):
      if len(str(filenames[i])):
        cond = ("id",str(fileids[i]))      
        dic  = {"name":str(filenames[i])}
        sqlobj.updaterecords(self.SQLTable,dic,cond,auth=0)
        control = 1
    if control:
      return self.qattach.results_qattach(sqlobj)
        
      
  
  
########################################################################        
################### FILE MANAGEMENT ####################################
########################################################################
  
  def fileManagement(self,request):
    # name,type,size,auth,archive
    SESSION = request.SESSION["FileManager"]
    userauth=SESSION["Userauth"]
    filter=SESSION["Filter"]
    groupid=SESSION["GroupID"]
    recno=SESSION["Recno"]
    return self.getAllfilesFromDB(groupid,recno,filter,userauth)
    
    
    
  def fileProperties(self,request):
    SESSION = request.SESSION["FileManager"]
    PropMode = SESSION["ViewMode"]
    if PropMode[0] == "properties":
      fileid= PropMode[1]
      file = self.getfilefromDB(fileid)
      if len(file): return file[0]
      else: return {}
    elif PropMode[0] == "new":
      return {}
    
    
  def fileRenameList(self,request):
    fileids  = request.get("files",[])
    allfiles = self.fileManagement(request)
    filelist = []
    for allfile in allfiles:
      if allfile.get("id","") in fileids:
        filelist.append(allfile)
    return filelist    
    
  
  
InitializeClass(kzFileManager)
