# -*- coding: utf-8 -*- 
from AccessControl import User
from AccessControl.SecurityManagement import *
from kzFields import *
import string
import sys,traceback

superuser = User.system

def getException(scriptname):
  etype, value, tb = sys.exc_info()
  exctype  = str(etype)
  excvalue = str(value)
  lines    = traceback.format_tb(tb)
  lineinfo = ""
  lines.reverse()
  for line in lines:
    if str(line).find("in %s" % scriptname) >= 0:
      lineinfo = str(line)
      break
  
  return "%s:%s:%s" % (exctype,excvalue,lineinfo) 
  

class kzScript:
  #Tanimlanan script'leri calistirir
  def __init__(self,kztable,tagid,scriptname):
    self.kztable    = kztable
    self.tagid      = tagid
    self.scriptname = scriptname
    self.script     = self.getscript(tagid,scriptname)
    
    self.params     = self.kztable.qref.getChilds(tagid,"param")
    self.error      = ""
    
  def getscript(self,tagid,scriptname):
    container = self.kztable.qref.getContainer(tagid)
    if scriptname in container.objectIds():
      return getattr(container,scriptname)
    else:
      return None   
    
  def calculate(self,const_params=[],param_values=[],superusermode=0):
    # Scriptin parametrelerini bulur ve hesaplar
    
    if self.script is None:
      self.error = ("file error","script %s for %s is not found" % (self.scriptname,self.tagid) )
      return ("","")
    
    else:
    
      variables = vars()
      for i in range(len(const_params)):
        variables[const_params[i]] = param_values[i]
        
      script_params_text = getattr(self.script,'_params')
      const_params_text  = string.join(const_params,",")
      control    = 0
      if len(script_params_text) == 0:
        params = const_params_text
        control = 1
      elif script_params_text.find(const_params_text) < 0:
        params = "%s,%s" % (const_params_text,script_params_text)
        control = 1
        
      if control:
        text = getattr(self.script,'_body')
        self.script.ZPythonScript_edit(params, text)  
        
      script_call_text = getattr(self.script,'call')  

      for param in self.params:
        paramname = param["paramid"]
        fieldid   = param["fieldrelation"]
        script_call_text = string.replace(script_call_text,
                                          '$%s$' % (paramname),
                                          "'%s'" % (fieldid) )
        # Parametre degerlerini hesapla
        if script_call_text.find('|%s|' % (paramname)) > -1:
          try:
            variables[paramname] = self.get_param(param)
            script_call_text = string.replace(script_call_text,'|%s|' \
                                              % (paramname),"%s" % (paramname) )
          except:
            self.error = ("get_param","%s %s" % (getException(self.scriptname),param) )
            return ("","")

      if superusermode:
        securitymanager = getSecurityManager()
        newSecurityManager(request={},user=superuser)     
      
      try: 
        kuzgun = eval("self.script(%s)" % (script_call_text) )
      except:
        except_string = str(sys.exc_type) +":"+str(sys.exc_info)
        self.error    = ("script error","<br />tagid=%s <br />params=%s <br />error= %s" \
                         % (self.tagid,script_call_text,getException(self.scriptname)) )
        kuzgun        = ("","")
      
      if superusermode:
        setSecurityManager(securitymanager)

      return kuzgun
    
  def get_param(self,param):
    # Parametre tipine g�bir deger olusturur ve dondurur
    if param["paramtype"] == "0":
      return self.get_param_0(param["fieldrelation"])
    if param["paramtype"] == "1":
      return self.get_param_1(param["fieldrelation"])
    else:  
      return self.get_param_2(param["fieldrelation"])
      
  def get_param_0(self,fieldid):
    # Ayni tablo i�deki alan degerini alir
    if fieldid[:fieldid.rfind("_")] == self.tagid[:self.tagid.rfind("_")]:
      return self.kztable.get_field(fieldid)
    else:
      return [-1,-1]
  
  def get_param_1(self,fieldid):
    # Ayni grup i�deki alan degerini alir
    tableid    = fieldid[:fieldid.rfind("_")]
    masterrecno= self.kztable.essentials.masterrecno
    # Kendi alt tablosu mu diye bakiyor
    if self.kztable.subtable != None:
      if self.kztable.subtable.tableID == tableid:
        masterrecno = self.kztable.essentials.recno
    # Alan �likleri aliniyor    
    field_attr = self.kztable.qref.getAttributes(fieldid)
    from kzFields import kzField
    field      = kzField(self.kztable,fieldid,field_attr)
    
    fields = []
    for name in field.names:
      fields.append(field.tagid+name)
    cond = ("masterrecno",int(masterrecno))  
    rec = self.kztable.qdata.sqlselectrecords(tableid,fields,cond=cond)

    result = []
    for r in rec:
      field.set(r)
      result.append(field.get_value())
    return result
  
  def get_param_2(self,fieldid):
    # T�n degerini alir
    tableid    = fieldid[:fieldid.rfind("_")]
    field_attr = self.kztable.qref.getAttributes(fieldid)
    from kzFields import kzField
    field      = kzField(self.kztable,fieldid,field_attr)
    
    fields = []
    for name in field.names:
      fields.append(field.tagid+name)
    rec = self.kztable.qdata.sqlselectrecords(tableid,fields)

    result = []
    for r in rec:
      field.set(r)
      result.append(field.get_value())
    return result
  
  
