# -*- coding: utf-8 -*- 

from kzFields  import *
from kzObjects import *
from kzScript import *

from DateTime import DateTime
from OFS.FindSupport import FindSupport
from Products.kzDBDesign.kzDBDesign import get_object

from AccessControl  import ClassSecurityInfo


def dictionary_add(dic1,dic2):
  # Iki dictionary'i birlestirir
  for key1 in dic2.keys():
    if not dic1.has_key(key1):
      dic1[key1] = dic2[key1]
    else:
      for key2 in dic2[key1].keys():
        if not dic1[key1].has_key(key2):
          dic1[key1][key2] = dic2[key1][key2]

          

class base_table:
  # Tablo nesnesinin base objesi
  
  security = ClassSecurityInfo()

  security.declarePublic("essentials") 
  def __init__(self,tableID):
    self.tableID               = tableID
    self.tableType             = ""
    self.tableCaption          = ""
    self.tableDescription      = ""
    self.tableMasterSummary    = 0
    
    self.errors          = []
    self.messages        = []
    
    self.essentials      = kzEssentials(self)
    self.fields          = []
    
    self.standartformbuttons = []
    self.formbuttons     = []

    self.mastertable     = None
    self.subtable        = None
    self.toptable        = None
    
    # Tablonun durumunu belirtir
    self.recordMode      = 0
    self.searchMode      = 0
    self.customerMode    = 0
    self.readonlyMode    = 0
    
    # Genel tarih formati
    self.dateFormat      = "dmy"
    
    # Siralama yapmak i� kullanilacak alanlar
    self.fieldsForOrder  = []
    self.setFieldsForOrder()

    # Dis erisimler
    self.qref            = None
    self.qdata           = None
    self.parent          = None
    
    self.Records         = kzRecords(tableID,self.fieldsForOrder,self.fieldsForOrder)
    
    self.tempattr        = {}
    
    
#########################################################################
######################### TEMP ATTRIBUTES    ############################
#########################################################################

  def clearTempAttributes(self):
    self.tempattr = {}
    
  def setTempAttributes(self):
    triggerobj = kzScript(self,self.tableID,"dynamicprops")
    triggerobj.calculate(["kzobject"],[self])
    
  security.declarePublic("setTempTitle")            
  def setTempTitle(self,value):
    self.tempattr["title"] = value
    
  security.declarePublic("getTitle")            
  def getTitle(self):
    return self.tempattr.get("title",self.tableCaption)
    
  security.declarePublic("setTempReadonly")            
  def setTempReadonly(self):
    self.tempattr["readonly"] = 1
    
  security.declarePublic("unsetTempReadonly")            
  def unsetTempReadonly(self):
    self.tempattr["readonly"] = 0
    
  security.declarePublic("isReadonly")            
  def isReadonly(self):
    return self.tempattr.get("readonly",self.readonlyMode)    
    
    
#########################################################################
######################### DEFAULT CONDS    ##############################
#########################################################################

  def getDefaultConds(self,cond=()):
    defaultconds = self.parent.getDefaultConds(self.tableID)
    if defaultconds:
      if cond:
        return (cond,'and',defaultconds)
      else:
        return defaultconds
    else:
      return cond  
    
  def setDefaultConds(self):
    cond = self.getDefaultConds()
    self.Records.defaultCond = cond
    
################################################################################

  def getErrors(self):
    err = self.errors
    self.errors=[]
    return err
  
  def getMessages(self):
    msg = self.messages
    self.messages=[]
    return msg

################################################################################
    
  def set_recno(self,auth=1):
    sqlobj = self.qdata.getsqlobject()
    cond = ('recno',int(self.essentials.recno) )
    cond = self.getDefaultConds(cond)
    sqlobj.selectrecords(self.tableID,['*'],cond=cond)
    rec = self.qdata.results_qdata(sqlobj)  
    if len(rec):
      self.set(rec[0])
      self.essentials.set(rec[0])
    else:
      self.reset()
      
  def set_new(self):
    for field in self.fields:
      field.setnew()
    self.essentials.set_new()
    self.calculate() 
    self.set_mode(1,0)
    
  def reset(self):
    for field in self.fields:
      field.reset()
    self.essentials.reset()
    self.set_mode(0,0)
    
  def refresh(self,request):
    self.essentials.update(request)
    self.set(request.form)
  
  def set(self,dic):
    for field in self.fields:
      field.set(dic)
    self.calculate() 
    
  def calculate(self):
    for field in self.fields:
      field.calculate() 
    
  def get(self):
    dic = {}
    for field in self.fields:
      temp = field.get()
      dictionary_add(dic,temp)
    return dic
  
#########################################################################
######################### SEARCHING    ##################################
#########################################################################  
  
  def prepareForSearching(self):
    for field in self.fields:
      field.prepareForSearching()
    self.essentials.prepareForSearching()  
    self.set_mode(0,1)
    
  def endSearching(self):
    for field in self.fields:
      field.endSearching()
    self.essentials.endSearching()  
    self.set_mode(0,0)
    
  def search(self,request):
    # Formdaki kisitlara g�arama yapar
    control= 0
    fields = []
    cond   = ()
    for field in self.fields:
      fields   = fields + field.getSearchableIds()
      cont,tempcond = field.getSearchableValues(request)
      if cont:
        if len(tempcond):
          if len(cond): cond = (cond,'and',tempcond)
          else: cond = tempcond
          if cont: control = 1
      
    if control:
        # Kayitlari se�        
        cond = self.getDefaultConds(cond)
        self.Records.setNewCond( cond )
        self.Records.reload(self.qdata)
        return 1
    else:
      return 0
        
################################################################################        
  
  def setFieldsForOrder(self):
    # Tablonun alanlarini tarayip uygun bir liste olusturur
    self.fieldsForOrder = []
    for field in self.fields:
      self.fieldsForOrder = self.fieldsForOrder + field.getFieldsForOrder()
    self.fieldsForOrder.append("recno")
    
  def get_tables(self):
    # Tablonun da icinde bulundugu grubun tum tablolarini d�r� 
    tables = []
    for table in self.parent.tables:
      if table.toptable == None:
        tables.append( ( table.tableID, table.tableType, table.subtable ) )
    return tables    
        
  
  def get_field(self,fieldid):
    # Tablonun icindeki fieldid alanini dondurur
    for field in self.fields:
      if field.attr["tagid"] == fieldid:
        storage1 = copy.deepcopy(field.storage1)
        storage2 = copy.deepcopy(field.storage2)
        return [storage1,storage2]
    return [-1,-1] 
  

  def getFieldByTagid(self,tagid):
    for field in self.fields:
      if field.tagid == tagid:
        return field
    return None  
  
  
  def set_mode(self,record,search):
    self.recordMode = record
    self.searchMode = search
    
   
    
  ## RECNO OPERATIONS  
    
  def getsequenceobj(self,tableid=None):
    if tableid is None:
      tableid = self.tableID
    try:  
      return tableid,self.qref.getContainer(tableid+"_sequence")
    except:
      table = self.qref.getAttributes(tableid)
      tableobj = get_object(self.qref,self.qdata,table)
      tableobj.setsequence()
      return tableid,self.qref.getContainer(tableid+"_sequence")
      
  def getnextrecno(self,tableid=None):
    tableid,sequence = self.getsequenceobj(tableid)
    try:
      return sequence.getNextValue(tableid)
    except:
      table = self.qref.getAttributes(tableid)
      tableobj = get_object(self.qref,self.qdata,table)
      tableobj.setsequence()
      return sequence.getNextValue(tableid)
  
  def getcurrecno(self,tableid=None):
    tableid,sequence = self.getsequenceobj(tableid)
    try:
      return sequence.getLastValue(tableid)
    except:
      table = self.qref.getAttributes(tableid)
      tableobj = get_object(self.qref,self.qdata,table)
      tableobj.setsequence()
      return sequence.getLastValue(tableid)
   
    
  def getlockobj(self,tableid=None):
    if tableid is None:
      tableid = self.tableID
    try:  
      return tableid,self.qref.getContainer(tableid+"_kzSharedResource")  
    except:
      container = self.qref.getContainer(tableid)
      container.manage_addProduct["kzSharedResource"].manage_addProduct("kzSharedResource","SharedResource")
      return tableid,self.qref.getContainer(tableid+"_kzSharedResource")  
  
  def checkRecnoLock(self,recno,tableid=None):
    tableid,lockobj = self.getlockobj(tableid)
    return lockobj.is_locked(recno)
    
  def lockRecno(self,recno,tableid=None):
    tableid,lockobj = self.getlockobj(tableid)
    return lockobj.get_lock(recno)
    
  def releaseRecno(self,recno,tableid=None):
    tableid,lockobj = self.getlockobj(tableid)
    lockobj.release_lock(recno)    
    
  ## END /RECNO OPERATIONS    
    
    
  
  def getRecordInfo(self):
    # Hangi alanlara bakacak
    recno = self.essentials.recno
    index,total = self.Records.getInfo(recno)
    return {"recno":recno,
            "index":index+1,
            "total":total,
            "recordmode":self.recordMode,
            "searchmode":self.searchMode}

  def getTableInfo(self):
    return {
      "id"    : self.tableID,
      "type"  : self.tableType,
      "description": self.tableDescription,
      "mastersummary" : self.tableMasterSummary
      }
  
  def getMasterSummary(self,container):
    fields = []
    info   = {}
    if self.mastertable:
      mastertableinfo = self.mastertable.getTableInfo()
      for field in self.mastertable.fields:
        if field.attr.get("quickview","") in [1,"1"]:
          fields.append( field.getFieldData(container) )
    return {"masterfields":fields,
            "mastertable":mastertableinfo}
  
      
  def getSubTableData(self,container): 
    subtable     = self.subtable
    if subtable:
      masterrecno  = subtable.toptable.essentials.recno
      subtable.essentials.masterrecno = masterrecno
      
      temptables = self.parent.tables
      temptableindex = self.parent.tableIndex
      self.parent.tables = [subtable]
      self.parent.tableIndex = 0
              
      fields = []
      for fieldid in subtable.fieldsForOrder:
          fields.append(fieldid)
          
      orderby= string.join(fields,',')    
      sqlobj = self.qdata.getsqlobject()
      cond=('masterrecno',masterrecno)
      cond = subtable.getDefaultConds(cond)
      sqlobj.selectrecords(subtable.tableID,['recno'],
                           cond=cond,orderby=orderby)
      rec = self.qdata.results_qdata(sqlobj)
  
      fieldinfo=[]
      fieldlist=[]
      for field in subtable.fields:
        if field.attr.get("quickview","0") in [1,"1"]:
          field.setTempAttributes()
          fieldinfo.append( field.getFieldInfo(container) )
          fieldlist.append(field)
     
      if not fieldlist:
        fieldlist = subtable.fields[:4]
        for field in fieldlist:
          field.setTempAttributes()
          fieldinfo.append( field.getFieldInfo(container) )
        
      records=[]
      for r in rec:
          subtable.essentials.recno = r["recno"]
          subtable.set_recno()
          record = []
          record.append(r["recno"])
          for field in fieldlist:
            record.append( field.getFieldStorages() )
          records.append(record)
          
      self.parent.tables = temptables
      self.parent.tableIndex = temptableindex
      
      return {
        "fieldinfo" : fieldinfo,
        "records"   : records,
        "errors"    : subtable.getErrors()
        }
 
#########################################################################  
########################### MENU ########################################
#########################################################################    
    
  def getStandartMenuButtons(self):
    # Tablonun durumuna uygun menu olusturur
    buttons=[]
    if not self.recordMode and \
       not self.searchMode and \
       not self.essentials.authMode == "Read" and \
       not self.readonlyMode and \
       self.essentials.masterrecno>0:
      buttons.append("ATTACH")

    if self.tableType!="d":
      if self.tableType in ["1","n"]:
        if self.essentials.masterrecno>0:
          buttons.append("GRID")
      else:
        buttons.append("GRID")
    buttons.append("HELP")
    return buttons
  
  def getDesignedMenuButtons(self):
    container = self.qref.getContainer(self.tableID)
    temp = FindSupport.ZopeFind(FindSupport(container), container, [],
                                ['kzFormButton'],search_sub=0) 
    return [tobj for tid,tobj in temp if (tid in self.formbuttons) and (self.controlActionResults(tobj.runCondition(self))) ]
  
  def getTableMenu(self):
    menubuttons = [bid for bid in self.getMenuButtons() if bid in self.formbuttons]
    return {"standart" : menubuttons,
            "designed" : self.getDesignedMenuButtons()}
          
 
#########################################################################  
#########################################################################
#########################################################################

  def controlActionResults(self,resultdic):
    result = resultdic["result"]
    error  = resultdic["error"]
    if not result:
      result,index = "", 1
    elif type(result) in [type(()),type([])]:
      result,index = result[0],result[1]>0
    elif result:
      result,index = result, 1
    else:
      result,index = "",1
    if error:
      if error[0] != 'file error':
        self.errors.append(error)
        index = 0
      else:
        index = 1
    elif index>0:
      if result:
        self.messages.append( ("table message",result) )
    else:
      if type(result) is type(""):
        self.errors.append( ("table error",result) )
      elif type(result) is type([]):
        self.errors.extend(result)
      
    return index
    
    
  def controlActions(self,tableid,action,dic):
    script = kzScript(self,tableid,action)
    result = script.calculate(["form_values"],[dic])
    return self.controlActionResults({"error" : script.error,
                                      "result": result})
  
  def controlRequired(self,tableid,dictionaries):
    dict = dictionaries.get(tableid,{})
    control = 1
    if dict:
      for field in self.qref.getChilds(tableid,'field'):
        fieldid = field["tagid"]
        fieldtitle = field["title"]
        if tableid != self.tableID:
          obj = eval("kzField_%s(tableid,field['tagid'],field)" % (field["type"]))
          obj.set(dict)
        else:
          obj = self.getFieldByTagid(fieldid)
        if obj:  
          if not obj.controlRequired():
            self.errors.append( ("Require Error","%s must be filled out!" % fieldtitle) )
            control = 0
    return control
          
  
  
  def beforeAction(self,dictionaries,tablelist,action):
    # Verileri veritabanina ekleyebilecek hale getir
    for table in tablelist:
      if not self.controlRequired(table,dictionaries):
        return 0
      if not self.controlActions(table,action,dictionaries):
        return 0
    return 1  
      
  def onAction(self,dictionaries,tablelist,action):
    # Verileri veritabanina ekleyebilecek hale getir
    for table in tablelist:
      self.controlActions(table,action,dictionaries)
      
  def runDesignedButtonActions(self,buttonname):
    if buttonname in [bid for bid in self.formbuttons if not bid in self.standartformbuttons]:
      container = self.qref.getContainer(self.tableID)
      if buttonname in container.objectIds():
        designedbuttonobj = getattr(container,buttonname)
        designedbuttonobj.setTableid(self.tableID)
        if self.controlActionResults(designedbuttonobj.runValidation(self)):
          self.controlActionResults(designedbuttonobj.runAction(self))
    

class kzTable_m(base_table):
  # Master tablo nesnesi
  
  def __init__(self,tableID):
    base_table.__init__(self,tableID)
    self.Records = kzRecords(tableID,self.fieldsForOrder,self.fieldsForOrder)
  
  def setMasterRecNo(self,masterRecNo):
    self.essentials.masterrecno = masterRecNo
    self.essentials.recno       = masterRecNo
    self.set_mode(0,0)
    
    
  def add(self,request):
    # Verileri tablo nesnesine ata
    self.set(request.form)
    recno = self.essentials.recno
    # Essential alanlari al
    self.essentials.add(request)
    # Tablonun da i�de bulundu�rubun tum tablolari bulunuyor(tableid,tabletype)
    tables = self.get_tables()
    # Verileri veritabanina ekleyebilecek hale getir
    dictionaries = self.get()
    
    # Sabitler olusturuluyor
    essentials={ "recno"       : int(self.essentials.recno),
                 "authority"   : int(self.essentials.authority),
                 "date"        : self.essentials.date,
                 "lastuser"    : self.essentials.lastuser,
                 "owner_name"  : self.essentials.owner_name }

    # Kayit Birakma
    if self.essentials.recno != recno:
      self.releaseRecno(recno)
      
    # Verileri tablolara yerlestir
    tablelist = []
    
    # Verileri master tabloya ekle
    dictionaries[self.tableID].update(essentials)
    tablelist.append(self.tableID)

    # Verileri diger tablolara ekle
    essentials.update({"masterrecno":self.essentials.recno})

    # Master tablonun alt tablosu icin
    if self.subtable is not None:
      if dictionaries.has_key(self.subtable.tableID):
        essentials.update( {"recno": int(self.getnextrecno(self.subtable.tableID)) } )
        dictionaries[self.subtable.tableID].update(essentials)
        tablelist.append(self.subtable.tableID)
        
    # T�lolar i� tara
    for tableID,tableType,subtable in tables:

      if tableType != "d" and tableID != self.tableID:
        
        # Alt tablo kontrolu
        subtablecontrol = 0
        if subtable != None:
          if dictionaries.has_key(subtable.tableID):
            subtablecontrol = 1
            
        # Tablo kontrolu    
        tablecontrol = 0    
        if not dictionaries.has_key(tableID):
          if subtablecontrol:
            dictionaries[tableID] = {}
            tablecontrol = 1
        else:
          tablecontrol = 1
          
        if tablecontrol:  
          essentials.update( {"recno": int(self.getnextrecno(tableID)) } )
          dictionaries[tableID].update(essentials)
          tablelist.append(tableID)

        if subtablecontrol:  
            essentials.update( {"recno": int(self.getnextrecno(subtable.tableID)) } )
            dictionaries[tableID].update(essentials)
            tablelist.append(tableID)
    
    if self.beforeAction(dictionaries,tablelist,"beforeinsert"):
      for table in tablelist:
        dic = dictionaries[table]
        self.qdata.sqlinsertrecords(table,dic)
        
      self.onAction(dictionaries,tablelist,"oninsert") 
      self.Records.add(self.essentials.recno,dictionaries[self.tableID])
      # Master kaydi set et
      self.setMasterRecNo(self.essentials.recno)
      # Tablonun durumunu belirt
      self.set_mode(0,0)  
          
          
  def save(self,request):
    #Gelenleri kaydeder
    if self.checkRecnoLock(self.essentials.recno):
      self.set_recno(self.essentials.recno)
    else:  
      # Verileri tablo nesnesine ata
      self.set(request.form)
      # Verileri veritaban�leyebilecek hale getir
      dictionaries = self.get()
      # Essential alanlari al
      self.essentials.update(request)
      # Verileri veritaban�a
      dictionaries[self.tableID]["authority"]   = int(self.essentials.authority)
      dictionaries[self.tableID]["lastuser"]    = self.essentials.lastuser
      dictionaries[self.tableID]["date"]        = self.essentials.date
      dictionaries[self.tableID]["owner_name"]  = self.essentials.owner_name
      dictionaries[self.tableID]["recno"]       = int(self.essentials.recno)
      if self.beforeAction(dictionaries,[self.tableID],"beforeupdate"):
        del dictionaries[self.tableID]["recno"]
        self.qdata.sqlupdaterecords(self.tableID,dictionaries[self.tableID],("recno",int(self.essentials.recno) ) )
        dictionaries[self.tableID]["recno"]     = int(self.essentials.recno)
        self.onAction(dictionaries,[self.tableID],"onupdate") 
        self.Records.update(self.essentials.recno,dictionaries[self.tableID])
        # Tablonun durumunu belirt
        self.set_mode(0,self.searchMode)
    
    
  def delete(self):
    # Kaydi siler
    if self.checkRecnoLock(self.essentials.recno):
      self.set_recno(self.essentials.recno)
    else:  
      dictionaries = self.get()
      # Sabitler olusturuluyor
      dictionaries[self.tableID]["authority"]   = int(self.essentials.authority)
      dictionaries[self.tableID]["lastuser"]    = self.essentials.lastuser
      dictionaries[self.tableID]["date"]        = self.essentials.date
      dictionaries[self.tableID]["owner_name"]  = self.essentials.owner_name
      dictionaries[self.tableID]["recno"]       = int(self.essentials.recno)
      # İşlem yapılan tablo listesi
      tablelist = []
      tablelist.append(self.tableID)
      if self.beforeAction(dictionaries,tablelist,"beforedelete"):
        # Kayit numarasi aliniyor
        recno = self.essentials.recno
        # Kayit Birakma
        self.releaseRecno(recno)
        # Bir onceki kayit
        self.essentials.recno = self.Records.delete(recno)
        self.setMasterRecNo(self.essentials.recno)
        self.set_recno()
        # sqlobj yarat
        sqlobj = self.qdata.getsqlobject()  
        # t�lolardan kaydi sil
        tables = self.get_tables()
        for tableID,tableType,subtable in tables:
    
          if tableType == "m":
            sqlobj.deleterecords(tableID,('recno',int(recno)))
            if subtable != None:
              sqlobj.deleterecords(subtable.tableID,('masterrecno',int(recno)))
              
          elif tableType == "d":
            sqlobj.deleterecords(tableID+"r",('master',int(recno)),auth=0)
    
          else:
            if subtable != None:
              rec = self.qdata.sqlselectrecords(tableID,['recno','masterrecno'],cond=('masterrecno',int(recno)))
              cond=()
              where_str = ""
              for r in rec:
                if len(cond):
                  cond = (cond,'or',('masterrecno',int(r["recno"])))
                else:
                  cond = ('masterrecno',int(r["recno"]))
              sqlobj.deleterecords(subtable.tableID,cond)    
            # 1-1 veya 1-N Tabloyu sil  
            sqlobj.deleterecords(tableID,('masterrecno',int(recno)))
            
        # Islemleri yap
        self.qdata.results_qdata(sqlobj)
        self.onAction(dictionaries,tablelist,"ondelete") 
      
      # Tablonun durumunu belirt
      self.set_mode(0,0)
    
    
  def navigation(self,direction='FIRST'):
    # Kayitlar �de dolasmayi saglar
    thisRecno = self.essentials.recno
    recno = self.Records.navigation(thisRecno,direction)
    if thisRecno != recno:
      self.releaseRecno(thisRecno)
    self.setMasterRecNo(recno)
    self.set_recno()
    
  def searchx(self,request):
    # Formdaki kisitlara g�arama yapar
    control= 0
    fields = []
    cond   = ()
    for field in self.fields:
      fields   = fields + field.getSearchableIds()
      cont,tempcond = field.getSearchableValues(request)
      if len(tempcond):
        if len(cond): cond = (cond,'and',tempcond)
        else: cond = tempcond
        if cont: control = 1
      
    if control:
        # Kayitlari se�        
        cond = self.getDefaultConds(cond)
        self.Records.setNewCond( cond )
        self.Records.reload(self.qdata)
        if self.Records.records:
          self.essentials.recno = self.Records.records[0]    
          self.setMasterRecNo(self.essentials.recno)  
          self.set_recno()
        else:
          self.essentials.recno = -1
          self.reset()  
        self.set_mode(0,0)
    else:
        self.setMasterRecNo(0)
        self.set_recno()
        self.set_mode(0,1)
        
          
  def getSubRecords(self):   
    return None
    
  def getMenuButtons(self):
    # Tablonun durumuna uygun menu olusturur                                     
    buttons = []
    if self.searchMode:
      buttons.append("SEARCH")
      buttons.append("END SEARCH")
    else:
      buttons.append("FIRST")
      buttons.append("PREV")
      buttons.append("NEXT")
      buttons.append("LAST")
      # Yetki kontrol�  
      buttons.append("auth")
      # Yeni kayit eklenecekse
      if self.recordMode:
        buttons.append("ADD")
      # Sadece Okuma  
      elif self.essentials.authMode == "Read" or self.readonlyMode:
        if self.essentials.authMode in ["Write","Read/Write"]:
          buttons.append("NEW")  
          buttons.append("MULTIROW")
        if self.essentials.recno > 0:
          buttons.append("TEMPLATE")
        buttons.append("null")  
      elif self.essentials.masterrecno < 0:
        buttons.append("NEW") 
        buttons.append("MULTIROW")
        buttons.append("null")
        buttons.append("START SEARCH")   
      else:
        buttons.append("SAVE")  
        buttons.append("REFRESH")  
        buttons.append("NEW") 
        buttons.append("MULTIROW")
        buttons.append("null")
        if self.essentials.authMode != "Write":
          buttons.append("DELETE")  
        buttons.append("null")
        buttons.append("START SEARCH") 
        buttons.append("TEMPLATE") 
    return buttons + base_table.getStandartMenuButtons(self)   
  
  
  
#########################################################################
########################## TABLE 1 ######################################
#########################################################################
  
class kzTable_1(base_table):
  # 1-1 iliskili tablo nesnesi
  
  def __init__(self,tableID):
    base_table.__init__(self,tableID)
    self.Records       = kzRecords(tableID,self.fieldsForOrder,self.fieldsForOrder)
  
  def setMasterRecNo(self,masterRecNo):
    # Master kayit numarasini set eder
    self.essentials.masterrecno = int(masterRecNo)
    self.Records.clear()
    if self.essentials.masterrecno < 0:
      self.reset()
    else:  
      cond = ('masterrecno',self.essentials.masterrecno)
      cond = self.getDefaultConds(cond)
      self.Records.setNewCond( cond )
      self.Records.reload(self.qdata,auth=0)
    
      if self.Records.records:
        if self.Records.dict[0]["authority"] > self.essentials.userAuth:
          self.Records.reset()
      else:
        self.essentials.add({})
        dic={"lastuser"   :self.essentials.lastuser,
             "authority"  :self.essentials.authority,
             "date"       :self.essentials.date ,
             "masterrecno":self.essentials.masterrecno,
             "recno"      :self.essentials.recno,
             "owner_name" :self.essentials.owner_name 
             }            
        self.qdata.sqlinsertrecords(self.tableID,dic)
        self.Records.reload(self.qdata)
        
    if self.Records.records:
      self.essentials.recno = self.Records.records[0]    
    else:
      self.essentials.recno = -1

    self.set_mode(0,0)
      
    
  def save(self,request):
    #Gelenleri kaydeder
    if self.checkRecnoLock(self.essentials.recno):
      self.set_recno(self.essentials.recno)
    else:  
      # Verileri tablo nesnesine ata
      self.set(request.form)
      # Essential'i g�e
      self.essentials.update(request)
      # Verileri veritaban�leyebilecek hale getir
      dictionaries = self.get()
      
      # Sabitler olusturuluyor
      masterrecno = int(self.essentials.masterrecno)
      recno       = int(self.essentials.recno)
      essentials  = { "authority"   : int(self.essentials.authority),
                      "date"        : self.essentials.date,
                      "lastuser"    : self.essentials.lastuser,
                      "owner_name"  : self.essentials.owner_name }
      
      masterid  = self.mastertable.tableID
      tablelist = []
      
      # Verileri master tabloya ekle
      if dictionaries.has_key(masterid):
        dictionaries[masterid].update(essentials)
        dictionaries[masterid].update({"recno":masterrecno})
        tablelist.append(masterid)
      
      # Verileri tabloya ekle
      dictionaries[self.tableID].update(essentials)
      dictionaries[self.tableID].update({"recno":recno,"masterrecno":masterrecno})
      tablelist.append(self.tableID)
      
      if self.beforeAction(dictionaries,tablelist,"beforeupdate"):
        
        if masterid in tablelist:
          del dictionaries[masterid]["recno"]
          self.qdata.sqlupdaterecords(masterid,dictionaries[masterid],
                                      cond=('recno',masterrecno))
          dictionaries[masterid].update({"recno":masterrecno})
        
        del dictionaries[self.tableID]["recno"]
        del dictionaries[self.tableID]["masterrecno"]
        self.qdata.sqlupdaterecords(self.tableID,dictionaries[self.tableID],
                                    cond=('recno',recno))
        
        # ON ACTION
        dictionaries[self.tableID].update({"recno":recno,"masterrecno":masterrecno})
        self.onAction(dictionaries,tablelist,"onupdate")  
          
        # Tablonun durumunu belirt
        self.set_mode(0,0)  
      
     
  def getSubRecords(self):   
    return None    
    
          
  def getMenuButtons(self):
    # Tablonun durumuna uygun menu olusturur
    buttons = []
    buttons.append("auth")
    if self.essentials.masterrecno > 0:
      if self.essentials.authMode != "Read" or self.readonlyMode:
        buttons.append("SAVE")
      buttons.append("REFRESH")    
      buttons.append("TEMPLATE") 
    return buttons + base_table.getStandartMenuButtons(self)   
  
  
#########################################################################
########################## TABLE N ######################################
#########################################################################    
  
class kzTable_n(base_table):
  # 1-N ili�kili tablo nesnesi
  
  def __init__(self,tableID):
    base_table.__init__(self,tableID)
    self.Records       = kzRecords(tableID,self.fieldsForOrder,["date"])
  
  def setMasterRecNo(self,masterRecNo):
    # Master kayit numarasini set eder
    self.essentials.masterrecno = int(masterRecNo)
    self.Records.clear()
    if self.essentials.masterrecno < 0:
      self.reset()
      self.essentials.recno = -1
      self.set_mode(0,0)
    else:  
      cond = ('masterrecno',self.essentials.masterrecno)
      cond = self.getDefaultConds(cond)
      self.Records.setNewCond( cond )
      self.Records.reload(self.qdata)
      
      if self.Records.records:
        self.essentials.recno = self.Records.records[0]
        self.set_recno()
        self.set_mode(0,0)
      else:
        self.set_new()
        self.set_mode(1,0)
   
    
  def add(self,request):
    # Verileri tablo nesnesine at
    self.set(request.form)
    # Essential'lari g�e
    recno = self.essentials.recno
    self.essentials.add(request)
    # Kayit Birakma
    if self.essentials.recno != recno:
      self.releaseRecno(recno)
    # Verileri veritaban�leyebilecek hale getir
    dictionaries = self.get()
    
    # Sabitler olusturuluyor
    masterrecno = int(self.essentials.masterrecno)
    recno       = int(self.essentials.recno)
    essentials  = { "authority"   : int(self.essentials.authority),
                    "date"        : self.essentials.date,
                    "lastuser"    : self.essentials.lastuser,
                    "owner_name"  : self.essentials.owner_name }
    
    masterid   = self.mastertable.tableID
    mtablelist = []
    ntablelist = []
    
    # Verileri master tabloya ekle
    if dictionaries.has_key(masterid):
      dictionaries[masterid].update(essentials)
      dictionaries[masterid].update({"recno":masterrecno})
      mtablelist.append(masterid)
    
    # Verileri tabloya ekle
    dictionaries[self.tableID].update(essentials)
    dictionaries[self.tableID].update({"recno":recno,"masterrecno":masterrecno})
    ntablelist.append(self.tableID)
    
    if self.beforeAction(dictionaries,mtablelist,"beforeupdate"):
      
      if self.beforeAction(dictionaries,ntablelist,"beforeinsert"):

        if masterid in mtablelist:
          del dictionaries[masterid]["recno"]
          self.qdata.sqlupdaterecords(masterid,dictionaries[masterid],
                                      cond=('recno',masterrecno))
        
        self.qdata.sqlinsertrecords(self.tableID,dictionaries[self.tableID])
        # Alt tablolari guncelle
        self.Records.add(recno,dictionaries[self.tableID])
        
        # ON ACTION
        if masterid in mtablelist:
          dictionaries[masterid].update({"recno":masterrecno})
          self.onAction(dictionaries,mtablelist,"onupdate")  
        
        self.onAction(dictionaries,ntablelist,"oninsert")     
          
        # Tablonun durumunu belirt
        self.set_mode(0,0)  
    

    
  def save(self,request):
    #Gelenleri kaydeder
    if self.checkRecnoLock(self.essentials.recno):
      self.set_recno(self.essentials.recno)
    else:  
      # Verileri tablo nesnesine ata
      self.set(request.form)
      # Verileri veritaban�leyebilecek hale getir
      dictionaries = self.get()
      # Essential'lari g�e
      self.essentials.update(request)
      
      # Sabitler olusturuluyor
      masterrecno = int(self.essentials.masterrecno)
      recno       = int(self.essentials.recno)
      essentials  = { "authority"   : int(self.essentials.authority),
                      "date"        : self.essentials.date,
                      "lastuser"    : self.essentials.lastuser,
                      "owner_name"  : self.essentials.owner_name }
      
      masterid  = self.mastertable.tableID
      tablelist = []
      
      # Verileri master tabloya ekle
      if dictionaries.has_key(masterid):
        dictionaries[masterid].update(essentials)
        dictionaries[masterid].update({"recno":masterrecno})
        tablelist.append(masterid)
      
      # Verileri tabloya ekle
      dictionaries[self.tableID].update(essentials)
      dictionaries[self.tableID].update({"recno":recno,"masterrecno":masterrecno})
      tablelist.append(self.tableID)
      
      if self.beforeAction(dictionaries,tablelist,"beforeupdate"):
        
        if masterid in tablelist:
          del dictionaries[masterid]["recno"]
          self.qdata.sqlupdaterecords(masterid,dictionaries[masterid],
                                      cond=('recno',masterrecno))
          dictionaries[masterid].update({"recno":masterrecno})
        
        del dictionaries[self.tableID]["recno"]
        del dictionaries[self.tableID]["masterrecno"]
        self.qdata.sqlupdaterecords(self.tableID,dictionaries[self.tableID],
                                    cond=('recno',recno))
        
        
        dictionaries[self.tableID].update({"recno":recno,"masterrecno":masterrecno})
        self.Records.update(recno,dictionaries[self.tableID])

        # ON ACTION
        self.onAction(dictionaries,tablelist,"onupdate")  
          
        # Tablonun durumunu belirt
        self.set_mode(0,0)  
      
   
    
  def delete(self):
    # Kayd�r
    if self.checkRecnoLock(self.essentials.recno):
      self.set_recno(self.essentials.recno)
    else:  
      tempdic = {
        "authority"  : int(self.essentials.authority),
        "lastuser"   : self.essentials.lastuser,
        "date"       : self.essentials.date,
        "owner_name" : self.essentials.owner_name,
        "recno"      : int(self.essentials.recno),
        "masterrecno": int(self.essentials.masterrecno)
        }
      dictionaries = self.get()
      dictionaries[self.tableID].update(tempdic)
      if self.beforeAction(dictionaries,[self.tableID],"beforedelete"):
        # Kaydi sil
        tableID = self.tableID
        recno   = self.essentials.recno
        # Kayit Birakma
        self.releaseRecno(recno)
        
        sqlobj  = self.qdata.getsqlobject()
        # Alt tablodan sil
        if self.subtable != None:
          sqlobj.deleterecords(self.subtable.tableID,("masterrecno",int(recno)))
    
        # 1-N Tablodan sil  
        sqlobj.deleterecords(tableID,("recno",int(recno)))
        self.qdata.results_qdata(sqlobj)
        # Alt kay� d�
        if self.essentials.masterrecno != -1:
          self.essentials.recno = self.Records.delete(recno)
          if self.essentials.recno == -1:
            self.set_mode(1,0)
          else:
            self.set_mode(0,0)
        else:
          self.essentials.recno = -1
          self.set_mode(0,0)
        # Yeni kayd�oya ekle    
        self.set_recno()
        dictionaries = self.get()
        dictionaries[self.tableID].update(tempdic)
        masterid = self.mastertable.tableID
        if dictionaries.has_key(masterid):
          dictionaries[masterid]["lastuser"] = self.essentials.lastuser
          dictionaries[masterid]["date"]     = self.essentials.date
          dictionaries[masterid]["recno"]    = self.essentials.masterrecno
          self.qdata.sqlupdaterecords(masterid,dictionaries[masterid],
                                   cond=("recno",int(self.essentials.masterrecno))) 
        # ON ACTION
        self.onAction(dictionaries,[self.tableID],"ondelete")  
      
      
 
  def getSubRecords(self): 
    if self.Records.records:
      return self.Records.dict
    else:
      return None  
  
  
  def getMenuButtons(self):
    # Tablonun durumuna uygun menu olusturur
    buttons = []
    buttons.append("auth")
    if self.essentials.masterrecno > 0:
      if self.essentials.authMode == "Read"  or self.readonlyMode:
        if len(self.Records) != 0:
          buttons.append("TEMPLATE") 
      elif len(self.Records) == 0:
        if self.recordMode:
          buttons.append("ADD")
        else:  
          buttons.append("NEW")
      elif self.recordMode: 
        buttons.append("ADD")
      else:
        buttons.append("SAVE")
        buttons.append("NEW")
        if self.essentials.authMode != "Write":
          buttons.append("DELETE")
        buttons.append("TEMPLATE") 
      buttons.append("REFRESH")    
    return buttons + base_table.getStandartMenuButtons(self)       
          
  
  
#########################################################################
#########################################################################
#########################################################################
########################## TABLE NN #####################################
#########################################################################        
  
class kzTable_nn(base_table):
  # N-N ili�kili tablo nesnesi
  
  def __init__(self,tableID):
    base_table.__init__(self,tableID)
    self.fieldsForOrder = ["caption","recno"]
    self.Records        = kzRecordsNN(tableID,tableID+"r",self.fieldsForOrder,self.fieldsForOrder )
  
  def setMasterRecNo(self,masterRecNo):
    # Master kayit numarasini set eder
    self.essentials.masterrecno = int(masterRecNo)
    self.Records.clear()
      
    cond = ('master',self.essentials.masterrecno)
    cond = self.getDefaultConds(cond)
    self.Records.setRCond( cond )
    self.Records.reload(self.qdata)
    
    if self.essentials.masterrecno < 0:
      self.reset()
      self.essentials.recno = -1
      self.set_mode(0,0)
    
    elif self.Records.records:
      self.essentials.recno = self.Records.records[0]
      self.set_recno()
      self.set_mode(0,0)
    else:
      self.set_new()
      self.set_mode(1,0)
      
   
  def new(self,recno,caption):
    # Tablodaki bir kaydi kendisine yeni olarak ekler
    self.essentials.recno = int(recno)
    self.set_recno()
    dictionaries = self.get()
    dictionaries[self.tableID].update({"_prompt_":caption})
    self.add(dictionaries[self.tableID])
    # Tablonun durumunu belirt
    self.set_mode(0,0)
  
  def add(self,request):
    # Verileri tablo nesnesine ata
    try: self.set(request.form)
    except: self.set(request)
    # Yeni kayt a�
    self.essentials.add(request)
    # Verileri veritaban�leyebilecek hale getir
    dictionaries = self.get()
    # Yeni kayd�l�arlan�      
    caption = request.get("_prompt_","caption")
              
    # Sabitler olusturuluyor
    masterrecno = int(self.essentials.masterrecno)
    recno       = int(self.essentials.recno)
    essentials  = { "authority"   : int(self.essentials.authority),
                    "date"        : self.essentials.date,
                    "lastuser"    : self.essentials.lastuser,
                    "owner_name"  : self.essentials.owner_name }
    
    masterid    = self.mastertable.tableID
    mtablelist  = []
    nntablelist = []
    
    # Verileri master tabloya ekle
    if dictionaries.has_key(masterid):
      dictionaries[masterid].update(essentials)
      dictionaries[masterid].update({"recno":masterrecno})
      mtablelist.append(masterid)
    
    # Verileri tabloya ekle
    dictionaries[self.tableID].update(essentials)
    dictionaries[self.tableID].update({"recno":recno,"masterrecno":masterrecno,"caption":caption})
    nntablelist.append(self.tableID)
    
    if self.beforeAction(dictionaries,mtablelist,"beforeupdate"):
      
      if self.beforeAction(dictionaries,nntablelist,"beforeinsert"):

        if masterid in mtablelist:
          del dictionaries[masterid]["recno"]
          self.qdata.sqlupdaterecords(masterid,dictionaries[masterid],
                                      cond=('recno',masterrecno))
        
        self.qdata.sqlinsertrecords(self.tableID,dictionaries[self.tableID])
        # NN tablonun referans tablosuna da kaydi ekle
        dic = {}
        dic["master"] = masterrecno
        dic["recno"]  = recno
        self.qdata.sqlinsertrecords(self.tableID+"r",dic) 
        # Alt tablolari guncelle
        self.Records.add(recno,dictionaries[self.tableID])
        
        # ON ACTION
        if masterid in mtablelist:
          dictionaries[masterid].update({"recno":masterrecno})
          self.onAction(dictionaries,mtablelist,"onupdate")  
        
        self.onAction(dictionaries,nntablelist,"oninsert")     
          
        # Tablonun durumunu belirt
        self.set_mode(0,0)  
    
          
              
  def save(self,request):
    # Gelenleri kaydeder
    if self.checkRecnoLock(self.essentials.recno):
      self.set_recno(self.essentials.recno)
    else:  
      # Verileri tablo nesnesine ata
      self.set(request.form)
      # Essential'i g�e
      self.essentials.update(request)
      # Verileri veritaban�leyebilecek hale getir
      dictionaries = self.get()
      caption = request.get("_prompt_","caption")
     
      # Sabitler olusturuluyor
      masterrecno = int(self.essentials.masterrecno)
      recno       = int(self.essentials.recno)
      essentials  = { "authority"   : int(self.essentials.authority),
                      "date"        : self.essentials.date,
                      "lastuser"    : self.essentials.lastuser,
                      "owner_name"  : self.essentials.owner_name }
      
      masterid  = self.mastertable.tableID
      tablelist = []
      
      # Verileri master tabloya ekle
      if dictionaries.has_key(masterid):
        dictionaries[masterid].update(essentials)
        dictionaries[masterid].update({"recno":masterrecno})
        tablelist.append(masterid)
      
      # Verileri tabloya ekle
      dictionaries[self.tableID].update(essentials)
      dictionaries[self.tableID].update({"recno":recno,"masterrecno":masterrecno,"caption":caption})
      tablelist.append(self.tableID)
      
      if self.beforeAction(dictionaries,tablelist,"beforeupdate"):
        
        if masterid in tablelist:
          del dictionaries[masterid]["recno"]
          self.qdata.sqlupdaterecords(masterid,dictionaries[masterid],
                                      cond=('recno',masterrecno))
          dictionaries[masterid].update({"recno":masterrecno})
        
        del dictionaries[self.tableID]["recno"]
        del dictionaries[self.tableID]["masterrecno"]
        self.qdata.sqlupdaterecords(self.tableID,dictionaries[self.tableID],
                                    cond=('recno',recno))
        
        
        dictionaries[self.tableID].update({"recno":recno,"masterrecno":masterrecno})
        self.Records.update(recno,dictionaries[self.tableID])

        # ON ACTION
        self.onAction(dictionaries,tablelist,"onupdate")  
          
        # Tablonun durumunu belirt
        self.set_mode(0,0)  
      
    
  
  def getrec(self,recno,caption):
    # Tablodaki bir kaydi kendisine baglar
    recno = int(recno)
    if not recno in self.Records.records:
      # Yeni kayit numarasini ayarla
      self.essentials.recno = int(recno)
      self.set_recno()
      dictionaries = self.get()
      
      # Sabitler olusturuluyor
      masterrecno = int(self.essentials.masterrecno)
      recno       = int(self.essentials.recno)
      essentials  = { "authority"   : int(self.essentials.authority),
                      "date"        : self.essentials.date,
                      "lastuser"    : self.essentials.lastuser,
                      "owner_name"  : self.essentials.owner_name }
      
      masterid  = self.mastertable.tableID
      tablelist = []
      
      # Verileri master tabloya ekle
      if dictionaries.has_key(masterid):
        dictionaries[masterid].update(essentials)
        dictionaries[masterid].update({"recno":masterrecno})
        tablelist.append(masterid)
      
      if self.beforeAction(dictionaries,tablelist,"beforeupdate"):
        
        if masterid in tablelist:
          del dictionaries[masterid]["recno"]
          self.qdata.sqlupdaterecords(masterid,dictionaries[masterid],
                                      cond=('recno',masterrecno))
          dictionaries[masterid].update({"recno":masterrecno})
          
        # Tablonun referans tablosuna kaydi ekle
        dic = {}
        dic["master"] = self.essentials.masterrecno
        dic["recno"]  = recno
        self.qdata.sqlinsertrecords(self.tableID+"r",dic)  
        # Alt kayitlara ekle
        dic = {}
        dic["caption"] = caption
        dic["recno"]   = str(recno)
        self.Records.add(recno,dic)
          
        # ON ACTION
        self.onAction(dictionaries,tablelist,"onupdate")  
          
        # Tablonun durumunu belirt
        self.set_mode(0,0)  
      
    
  def delete(self):
    # Kaydi siler
    if self.checkRecnoLock(self.essentials.recno):
      self.set_recno(self.essentials.recno)
    else:  
      dictionaries = self.get()
      dictionaries[self.tableID]["authority"]   = int(self.essentials.authority)
      dictionaries[self.tableID]["lastuser"]    = self.essentials.lastuser
      dictionaries[self.tableID]["date"]        = self.essentials.date
      dictionaries[self.tableID]["owner_name"]  = self.essentials.owner_name
      dictionaries[self.tableID]["recno"]       = int(self.essentials.recno)
      if self.beforeAction(dictionaries,[self.tableID],"beforedelete"):
        # Kay� sil
        tableID = self.tableID
        recno   = self.essentials.recno
        cond    = (("master",int(self.essentials.masterrecno)),'and',("recno",int(recno)))
        self.qdata.sqldeleterecords(tableID+"r",cond=cond,auth=0)
        # Alt kay� d�
        if self.essentials.masterrecno != -1:
          self.essentials.recno = self.Records.delete(recno)
        else:
          self.essentials.recno = -1
        # Yeni kayd�oya ekle    
        self.set_recno()
        dictionaries = self.get() 
        # Verileri master tabloya ekle
        masterid = self.mastertable.tableID
        if dictionaries.has_key(masterid):
          dictionaries[masterid]["lastuser"] = self.essentials.lastuser
          dictionaries[masterid]["date"]     = self.essentials.date
          self.qdata.sqlupdaterecords(masterid,dictionaries[masterid],
                                   cond=('recno',int(self.essentials.masterrecno)))
        # ON ACTION
        self.onAction(dictionaries,[self.tableID],"ondelete")  
        # Tablonun durumunu belirt
        self.set_mode(0,0)
      
  def getUnRelatedRecords(self):
    # NN tablonun kaytlarn listeler ancak master kayda iliskin
    # alt kaytlar listelenmez
    masterrecno = self.essentials.masterrecno
    # Iliskili kayitlari getiren sorgu
    sqlobj  = self.qdata.getsqlobject()
    cond    = ("master",masterrecno)
    sqlobj.selectrecords("%sr"%self.tableID,["recno"],cond=cond)    
    cond = ('recno','not in',[ '(%s)' % sqlobj.sql[0] ] )
    return self.qdata.sqlselectrecords(self.tableID,"*",cond=cond)
  
  def getRecordCaption(self):
    recno = self.essentials.recno
    index,size = self.Records.getInfo(recno)
    if index>=0 and index<size:
      return self.Records.dict[index].get('caption','')
    else:
      return ''

    
  def delete_from_db(self):
    # Kaydi veritabanindan siler
    tableID = self.tableID
    recno   = self.essentials.recno
    sqlobj  = self.qdata.getsqlobject()
    if self.subtable != None:
      sqlobj.deleterecords(self.subtable.tableID,("masterrecno",int(recno)))
    sqlobj.deleterecords(tableID,("recno",int(recno)))
    self.qdata.results_qdata(sqlobj)
    self.delete()
    
  def getSubRecords(self): 
    if self.Records.records:
      return self.Records.dict
    else:
      return None  
  
  def getMenuButtons(self):
    # Tablonun durumuna uygun menu olusturur
    buttons = []
    buttons.append("auth")
    if self.essentials.masterrecno < 0:
      pass
    else:
      if self.essentials.authMode == "Read" or self.readonlyMode:
        buttons.append("TEMPLATE") 
      else:
        if self.essentials.recno < 0:
          buttons.append("NEW")
          buttons.append("GET")
        elif self.recordMode: 
          buttons.append("ADDNN")
          buttons.append("GET")
        elif len(self.Records) == 0:
          buttons.append("ADDNN")
          buttons.append("GET")
        else:  
          buttons.append("SAVENN")
          buttons.append("NEW")
          if self.essentials.authMode != "Write":
            buttons.append("DELETENN")
          buttons.append("GET")
          buttons.append("TEMPLATE") 
      buttons.append("REFRESH")    
    return buttons + base_table.getStandartMenuButtons(self)           
      
InitializeClass(base_table)