from Globals import InitializeClass
from AccessControl import ClassSecurityInfo,Permissions
from Products.CMFCore import CMFCorePermissions
from Products.ZSQLMethods import SQL
from OFS.FindSupport import FindSupport        
from Products.CMFCore.PortalFolder import PortalFolder
import functions
import string
from Dimension import Dimension
from DateTime import DateTime
#----------------------------------------------------------------------#
factory_type_information = ( { 'id'             : 'Olap',
                               'meta_type'      : 'Olap',
                               'description'    : """\
                               Olap objects can be embedded in Portal documents.""",
                               'content_icon'           : 'olap.gif',
                               'product'        : 'kzOlap',
                               'factory'        : 'addOlap',
                               'immediate_view' : 'olap_edit_form',
                               'filter_content_types': 1,                              
                               'allowed_content_types':('Dimension',),
                               'actions'        :( 
                                   { 'id'            : 'view' 
                                     , 'name'          : 'View'
                                     , 'action'        : 'olap_view'
                                     , 'permissions'   : (CMFCorePermissions.View,)
                                     , 'category'      : 'object'
                                     },
                                   { 'id'              : 'contents' 
                                     , 'name'          : 'Contents'
                                     , 'action'        : 'folder_contents'
                                     , 'permissions'   : (CMFCorePermissions.View,)
                                     , 'category'      : 'object'
                                     }
                                   , { 'id'            : 'edit'
                                       , 'name'          : 'Edit'
                                       , 'action'        : 'olap_edit_form'
                                       , 'permissions'   : (CMFCorePermissions.ManageProperties,)
                                       , 'category'      : 'object'
                                       }
                                   )
                               }
                             ,
                             )


def addOlap(self,id,title=None):
    "add a new  object"
    rObj = Olap(id,title)
    self._setObject(id,rObj)
    

#----------------------------------------------------------------------#
class Olap(PortalFolder):
    "Kuzgun Olap Object"
    meta_type = 'Olap'
    portal_type = 'Olap'
    content_meta_types = 'Olap'
    manage_options = PortalFolder.manage_options
    #security = ClassSecurityInfo()
    
    def __init__(self,id,title=None):
        "initialize for Olap object"       
        PortalFolder.__init__(self,id,title)        
        self.connection = None
        self.tables = None
        self.group = None
        self.field = None        
        self.errors = {}
        self.select_x = None
        self.select_y = None
        self.select_z = None
        # --- create dimension -- #
        #self.invokeFactory(id='x',title="",type_name='Dimension')
        x = Dimension('x','X - Boyutu','')
        y = Dimension('y','Y - Boyutu','')
        z = Dimension('z','Z - Boyutu','')
        self._setObject('x',x)
        self._setObject('y',y)
        self._setObject('z',z)
        self.setConnection("Archieve")
        
    def fieldType(self):        
        return self.aq_inner.aq_parent.RefData.getAttributes(self.field)['type']
        
        
        
    def Table(self,relative_path=None):
        if relative_path is None:
            if self.field is not None:
                table = string.join(string.split(self.field,'_')[:-1],'_')
                return table
            #fi
        else: # get sub object table name
            dimension = getattr(self,relative_path)            
            table = string.join(string.split(dimension.field,'_')[:-1],'_')
            return table
            
    def setId(self,id):
        try: del self.errors['id']
        except: pass
        try:
            self._checkId(id)
            self.id = id
        except:
            self.errors['id'] = 'The id "%s" is reserved.' %(id)
            
            
            
            
    def allowedContentTypes(self):
        """
            List type info objects for types which can be added in
            this folder.
        """
        
        if len(self._objects) >= 3:
            return []
        
        return PortalFolder.allowedContentTypes(self)
    
    
    def setConnection(self,connection="Archieve"):
        #self.validate_connection(connection)
        self.connection = SQL.SQL('connection','title',connection,'query','<dtml-var query>')
        
    def connectionID(self):
        if self.connection != None and type(self.connection) != type('string'):
            connection_id = self.connection.connection_id
            return connection_id
        return self.connection
   
    def setGroup(self,group):
        try: del self.errors['group']
        except: pass
        try:
            self.validate_group(group)
            self.group = group
        except:
            self.errors['group'] = 'error_message_invalid_kuzgun_group'
        
            
    def setField(self,field):
        try:del self.errors['field']
        except: pass        
        try:
            self.check_field(field)
            self.field = field
        except:
            self.errors['field'] = 'error_message_invalid_field'
            
    def check_field(self,field):
        
        for f in self.getFields():
            if f['id'] == field :
                return 
            #fi
        #endfor
        raise
#----------------------------------------------------------------------#

    def SQLConnectionIDs(self):
        "return database connection ids"
        return SQL.SQLConnectionIDs(self.aq_inner.aq_parent)
    
    def getAllGroups(self):
        result = []
        groups = self.aq_inner.aq_parent.RefData.getAllGroups('g0',restriction=['isleaf','==','T'] )
        for group in groups:
            result.append({'id':group['tagid'],'title':group['title']})                        
        #endfor    
        return result
    
    def getTables(self):
        "return tables of group"
        tables = []
        if self.group != None :            
            for table in self.aq_inner.aq_parent.RefData.getAllTables(self.group):
                if table['type'] == 'm' or table['type'] == '1':
                    tables.append({'id':table['tagid'],'title':table['title']})            
            #endfor
        #fi
        return tables
    def getTableFields(self,table,field_type=None):
        "return fields for table"
        result = []
        tableid = table['id']
        if field_type is not None:
            fields = self.aq_inner.aq_parent.RefData.getChilds(tableid,'field',['type','==',field_type])
        else:
            fields = self.aq_inner.aq_parent.RefData.getChilds(tableid,'field')
        #fi
        for field in fields:
            result.append({'id':field['tagid'],'title':table['title']+'/'+field['title']})
        #end for
        return result 
    
    def getSelectedFields(self): # !!!!!!!!!!!!!!!!!!!!!!!!!!!!
        "return selected fields in other dimension"
        dimensions = self.objectValues('Dimension')
        fields = []
        for dimension in dimensions:
            #if dimension.id is not self.id:
            fields.append(dimension.field)
        #endfor
        return fields
    
    def getFields(self):
        "return selectable fields" 
        result = []
        selected_fields = self.getSelectedFields()
        for table in self.getTables():
            for field in self.getTableFields(table,getattr(self,'field_type',None)):
                if field in selected_fields:
                    continue
                #fi                
                result.append(field)
            #endfor
        #endfor
        return result
    def getFunctions(self):
        result = []
        for f in functions.__all__ :
            #f = eval('functions.'+f_name) 
            f_rec = {'id':f.__name__}
            if f.__doc__ != '' and f.__doc__ != None:
                f_rec['title'] = f.__doc__
            else:
                f_rec['title'] = f.__name__
            #fi
            result.append(f_rec) # register function
        #endfor
        return result
           
    def setFunction(self,function):
        try:del self.errors['function']
        except:pass
        try:
            self.check_function(function)
            self.function = function
        except:
            if function == 'None': self.errors['function'] = 'error_message_function_required'
            else:self.errors['function'] = 'error_message_function_is_not_defined'
        
    def check_function(self,function):
        for f in self.getFunctions():
            if f['id'] == function:
                return
            #fi
        #endfor
        raise
    
    def getSelections(self):
        "return Olap options"
        result = []
        for dimension in self.objectValues('Dimension'):
            sel_dict = {'id':dimension.id,
                        'values':dimension.getSelections()}
            result.append(sel_dict)
        #endfor
        return result
        
    def setQuery(self,REQUEST=None):
        "set x,y,z dimension values"
        form = REQUEST.form
        print REQUEST.keys()
        
    def createQuery(self,REQUEST=None):
        "create new SQL query for this state"
        form = REQUEST.form
        try:
            del form['createQuery']
        except: pass
        
        #self.renameObjects()
        self.select_z = form['z']
        self.select_y = form['y']
        self.select_x = form['x']
        #-----------------------------------#
        if len(self._objects) == 2:
            self.tables = self.viewTable()
        elif len(self._objects) == 3:
            self.tables = self.view3DTable()
        else :
            return None

        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/olap_view')
        
        
        
    def renameObjects(self):
        "rename dimensions name with (x,y,z)"
        return
        nameSpace = ['x','y','z']
        ids = []
        for obj in self.objectValues('Dimension'):
            if obj['id'] in nameSpace:
                nameSpace.remove(obj['id'])
            else:
                ids.append(obj['id'])
            #fi
        #endfor
        
        for id in ids:
            self.manage_renameObject(id,nameSpace[0])
            setattr(self,'select_'+nameSpace[0],nameSpace[0])
            nameSpace = nameSpace[1:]
        #endfor
            
            
    def viewTable(self):
        "create 2D olap cube and view table(s)"
        pass
    def createSQLQuery(self,Query=None):
        field = self.field
        if self.fieldType() in ['i','m']:
            field += '_0'
            
        query = 'SELECT %s FROM %s' %(field,self.Table())
        where = ''
        for condition in Query:            
            selections = condition['selections']
            field = condition['field']
            if selections != '':
                sub_query = ''
                selections = string.split(selections,',')
                for selection in selections:
                    if selection == '':
                        continue
                    if string.find(selection,'-') != -1:
                        selection = selection.split('_')
                        if selection[1] != '':
                            if selection[0] > selection[1]:
                                selection[0],selection[1] = selection[1],selection[0]
                            #fi
                            expr = "(%s >= '%s' and %s <= '%s'" %(field,selection[0],field,selection[1])
                        else:
                            expr = "%s='%s'" %(field,selection[0])
                        #fi
                    elif string.find(selection,'*') != -1:                        
                        expr = "%s LIKE '%s'" %(field,selection.replace('*','%'))
                    else:
                        expr = "%s='%s'" %(field,selection)
                    #fi
                    if sub_query == '':
                        sub_query = expr
                    else:
                        sub_query += ' or %s' %(expr)
                    #fi
                #endfor
                if where == '':
                    where = '(%s)' %(sub_query)
                else:
                    where += ' and (%s)' %(sub_query)
                #fi
            #fi
        #endfor
        return query +' WHERE '+where 
                    
                    
    def createTable(self,xQueries,yQueries,zQuery=None):
        table = []
        for xQuery in xQueries:
            row = []
            for yQuery in yQueries:
                query = self.createSQLQuery(xQuery+yQuery+zQuery)                
                result = self.connection(query=query)
                result = result.tuples()
                value = self.calculate(result)
                row.append(value)
            #endfor
            table.append(row)
        #endfor
        return table
           
    
    def view3DTable(self):
        try:
            z_dimension = getattr(self,'z')
            z_queries   = z_dimension.queries(self.select_z)
            y_dimension = getattr(self,'y')
            y_queries   = y_dimension.queries(self.select_y)
            x_dimension = getattr(self,'x')
            x_queries   = x_dimension.queries(self.select_x)
            tables = []
            for z_query in z_queries:
                tables.append(self.createTable(x_queries,y_queries,z_query))
            #endfor
            return tables
        except AttributeError:
            self.reset()
            return self.view3DTable()
        else:
            pass
      
        
        
        
        
        
    def reset(self):
        self.select_x = ''
        self.select_y = ''
        self.select_z = ''
        
        
    def calculate(self,sqlResult):
        list = tupleList2List(sqlResult)
        f = getattr(functions,self.function)
        result = f(list)
        return result
        
        
    def Tables(self):
        "view Olap"
        self.renameObjects()
        return self.tables
        
        
        
        
    def Titles(self,dimension):
        try:
            obj = getattr(self,dimension)
            select = getattr(self,'select_%s'%(dimension))            
            path = select.split('/')
            for node in path:
                obj = getattr(obj,node)
            #endfor
            titles = []
            objects = obj.objectValues()
            if objects:
                for node in objects:
                    titles.append(node.title)
                #endfor
            else:
                titles.append(obj.title)
            #fi
            return titles
        except:
            pass                  
                                             
    def index_html(self,REQUEST=None):
        "return Olap_view"
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url()+'/olap_view')
#----------------------------------------------------------------------#    
    def validate_connection(self,connection=None):
        try: del self.errors['connection']
        except: pass
        
        for connection_id in self.SQLConnectionIDs():
            if connection_id[0] == connection:
                return
            #fi
        #endfor
        
        self.errors['connection'] = 'error_message_invalid_database_connection'
   
    def validate_group(self,group=None):               
        for g in self.getAllGroups():
            if group == g['id']:
                return 
            #fi
        #end for
        raise
        
   
#----------------------------------------------------------------------#        
    def edit(self,id,title,description,group,field,function,RESPONSE=None):
        "eee"        
        self.setId(id)
        self.setTitle(title)
        self.setDescription(description)
        #self.setConnection(connection)
        self.setGroup(group)
        self.setField(field)
        self.setFunction(function)
        if RESPONSE is not None :
            self.errors['test'] = 'ddd'
            if self.errors == {}:        
                RESPONSE.redirect(self.absolute_url()+'/folder_contents')
            else:
                RESPONSE.redirect(self.absolute_url()+'/olap_edit_form')
       
        
    def getTitle(self,path):
        path = path.split('/')
        obj = self.aq_inner        
        for node in path:
            obj = getattr(obj,node)
        #endfor
        return obj.title_or_id()
    
    
    def chartActive(self):        
        try:
            import gdchart
            return 1
        except ImportError:
            return 0
        
    def viewBarChart(self,REQUEST=None):
        "fgdfg"
        import gdchart  
        if self.chartActive():
            form = REQUEST.form
            if form.has_key('viewBarChart'):
                aboutBarChart = dictionaries(form['viewBarChart'])
                table_no = int(aboutBarChart['table_no'])
                if aboutBarChart.has_key('row'):
                    row = int(aboutBarChart['row'])
                    data = self.tables[table_no][row]
                    labels = self.Titles('y')
                    title = self.Titles('x')[row]
                    
                elif aboutBarChart.has_key('column'):
                    pass
                else:
                    raise
                #fi
                barchart = gdchart.Bar3D()
                barchart.width = 500
                barchart.height = 500
                barchart.ext_color = ["red", "green", "yellow", "orange", "blue"]
                barchart.title = title
                barchart.title_font = '/usr/share/fonts/default/trueType/arial.ttf'
                barchart.setData(data)
                barchart.setLabels(labels)
                fileName = 'barchart-'+str(float(DateTime())).replace('.','_')+'.png'
                filePath = '/tmp/'+fileName
                barchart.draw(filePath)
                file = open(filePath)
                self.aq_inner.aq_parent.temp_folder.manage_addProduct['OFSP'].manage_addImage(fileName,file,title='auto generate from Olap Product')
                zope_path='/temp_folder/'+fileName
                
                
                REQUEST['RESPONSE'].redirect(self.absolute_url()+'/olap_view?chart=%s' %(zope_path))
                
    
    def viewPieChart(self,REQUEST=None):
        "dfsdf"
        import gdchart
        
        
        if self.chartActive():
            form = REQUEST.form
            if form.has_key('viewPieChart'):
                aboutPieChart = dictionaries(form['viewPieChart'])
                table_no = int(aboutPieChart['table_no'])
                if aboutPieChart.has_key('row'):
                    row = int(aboutPieChart['row'])
                    data = self.tables[table_no][row]                 
                    labels = self.Titles('y')
                    title = self.Titles('x')[row]
                    
                elif aboutPieChart.has_key('column'): # column graphic
                    pass
                else:
                    raise
                #fi
                # tum veriler 0 ise chart kitleniyor
                if data.count(0) != len(data):
                    pieChart = gdchart.Pie3D()
                    pieChart.width = 500
                    pieChart.height = 500
                    pieChart.title = title
                    #pieChart.ytitle = ytitle
                    pieChart.color = ["red", "green", "yellow", "orange", "blue"]
                    #piechart.ytitle_font = '/usr/share/fonts/default/trueType/arial.ttf'
                    pieChart.title_font = '/usr/share/fonts/default/trueType/arial.ttf'
                    
                    
                    #pieChart.explode = []
                    pieChart._data = tuple(data)
                    pieChart._datalen = len(data)
                    #pieChart.setData(data)
                    pieChart.setLabels(labels)
                    fileName = 'piechart-'+str(float(DateTime())).replace('.','_')+'.png'
                    filePath = '/tmp/'+fileName
                    pieChart.draw(filePath)
                    file = open(filePath)
                    self.aq_inner.aq_parent.temp_folder.manage_addProduct['OFSP'].manage_addImage(fileName,file,title='auto generate from Olap Product')
                    zope_path='/temp_folder/'+fileName
                    s= 1
                    #REQUEST.set('chart',zope_path)
                    REQUEST['RESPONSE'].redirect(self.absolute_url()+'/olap_view?chart=%s' %(zope_path))
                #fi
                
    
def dictionaries(dic_str):
    dict = {}
    key_valueList = dic_str.split(',')
    for key_value in key_valueList:
        key,value = key_value.split(':')
        dict[key] = value
    #endfor
    return dict
def tupleList2List(sqlResult):
    "convert tuple list to list"
    result = []
    for t in sqlResult:
        result.append(t[0])
    #endfor
    return result
InitializeClass(Olap)        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
