from Products.CMFCore.utils import UniqueObject 
from OFS.SimpleItem import SimpleItem 
from Globals import InitializeClass 
from AccessControl import ClassSecurityInfo,Permissions
from kzManageProducts import kzManageProducts
import copy
import string

ACCESS_PROTECTED = Permissions.view


###############################################################
## container.manage_addProduct['kzUserManagement'].manage_addTool('kzUserManagement')


class kzUserManagement (UniqueObject, SimpleItem): 
  """ kzUserManagement  .... """ 
  id         = 'kzUserManagement' 
  meta_type  = 'kzUserManagement'
  plone_tool = 1 

  security = ClassSecurityInfo()
  
  def __init__(self):
    # Constructor
    self.auth_tableID        = "auth"
    self.auth_userID         = "userid"
    self.auth_nodeID         = "nodeid"
    self.auth_levelID        = "level"
    self.auth_modeID         = "mode"
    
    self.modules_tableID     = "kzmodules_allowed"
    self.modules_userID      = "userid"
    self.modules_users       = "users"
    self.modules_desicion    = "desicion"
    self.modules_application = "application"
    self.modules_report      = "report"
    
    self.kzAuthSqlID = 'kzAuthSQL'
    self.kzRefDataID = 'RefData'

    
  def getMainModules(self):
    return {"users":["users"],
            "desicion":["desicion"],
            "application":["menu","form","query"],
            "report":[]
            }
    
  def security(self,kk):
    return "checked"
  
  def listmembers(self):
    return []
    
      
  ###############################################################
  ###############################################################
  ## USER AUTH PUBLIC METHODS ###################################
  ###############################################################
  
  def getAuth(self,username='',nodename=''):
    # username'in nodename'deki yetkisini dondurur
    if nodename in ['',None] or username in ['',None]:
      return -1,"Read"
    else:
      qdata = self.getDBObj()
      return qdata.getAuth(username,nodename,mode=1)
          
  ###############################################################
  ###############################################################
  ## USER MODULE PUBLIC METHODS #################################
  ###############################################################    
    
  def getUserModules(self,username,no_recurse=0):
    # username icin izin verilen modulleri veritabanindan sorgular
    qdata      = self.getDBObj()
    sqlobj     = qdata.getsqlobject()
    usergroups = [str(username)]
    cond       = (self.modules_userID,str(username))
    user       = self.acl_users.getUser(str(username))
    if user and not no_recurse:
      groups = user.getGroups()
      usergroups += list(groups)
      for group in groups:
        cond = (cond, 'or', (self.modules_userID,str(group)) )
    sqlobj.selectrecords(self.modules_tableID,["*"],cond=cond,auth=0)
    try:			
      rec = qdata.results_qauth(sqlobj)
    except:
      self._createModulesTable()
      rec = []
      
    def findUserRecord(records,userid_field,userid_value):
      for i in range(len(records)):
        if records[i].get(userid_field,'') == userid_value:
          return i
      return -1  
      
    for u in usergroups:
      index = findUserRecord(rec, self.modules_userID, str(u) )
      if index >= 0:
        return rec[index]
    return {}     
  
  
  def getAllowedModules(self):
    # Sistemdeki kullanicinin yetkisi olan modullerini dondurur
    if self.aq_parent.portal_membership.isAnonymousUser():
      return []
    else:
      user   = self.aq_parent.portal_membership.getAuthenticatedMember() 
      if 'Manager' in user.getRolesInContext(self.acl_users):
        return self.getMainModules().keys()
      else:
        username = str(user)
        allowedmodules_dic  = self.getUserModules(username)
        allowedmodules_list = []
        for module in self.getMainModules().keys():
          if allowedmodules_dic.get(module,0):
            allowedmodules_list.append(module)
        return allowedmodules_list
      
      
  def getAllowedObjects(self,allowed_modules):    
    temp = []
    mainmodules = self.getMainModules()
    for allowed_module in allowed_modules:
      temp += mainmodules.get(allowed_module,[])
    return temp
  
  
  def getModuleObject(self,module):
    allowed_modules = self.getAllowedModules()
    allowed_objects = self.getAllowedObjects(allowed_modules)
    user = self.aq_parent.portal_membership.getAuthenticatedMember() 
    errtype = ""
    if module in allowed_objects:
      obj  = kzManageProducts(self.aq_parent,str(user),self.kzRefDataID,self.kzAuthSqlID)
      object  = obj.getModuleObject(module)
      if not object:
        errtype = "existance"
    else:
      object  = None
      errtype = "authority"
    return {"object":object,
            "errtype":errtype
            }
  

  def getDBObj(self):
    obj = getattr(self.aq_parent,self.kzAuthSqlID,None)
    if obj: return obj
    else: raise "%s named kzAuthSQL object not found" % self.kzAuthSqlID


  def getRefObj(self):
    self.kzRefDataID = 'RefData'
    obj = getattr(self.aq_parent,self.kzRefDataID,None)
    if obj: return obj
    else: raise "%s named kzRefData object not found" % self.kzRefDataID  
    
    
  ###############################################################
  ###############################################################
  ## USER GROUP PUBLIC METHODS ##################################
  ###############################################################
  
    
    
  ###############################################################
  ###############################################################
  ## PRIVATE METHODS ############################################
  ###############################################################      
      
  def process(self,REQUEST):
    
    self._select_action(REQUEST)
    self._auth_action(REQUEST)
    self._module_action(REQUEST)
    self._setmode(REQUEST)

  
  def _select_action(self,REQUEST):
    buttonname  =  REQUEST.get('buttonname','')
    buttonvalue =  REQUEST.get('buttonvalue','')
   
    if buttonname == "node":
      formframe   =  REQUEST.get('formframe','')
      if formframe == "usertree":
        REQUEST.set("user_selected",buttonvalue)
    
  
  def _module_action(self,REQUEST):
    AuthModulesMode = REQUEST.get('AuthModulesMode','')

    if AuthModulesMode == "modules":
      buttonname      = REQUEST.get('buttonname','')
            
      if buttonname == "save":
        UserGroupList = REQUEST.get('ChkUserGroup',[])
        users         = REQUEST.has_key("users")
        desicion      = REQUEST.has_key("desicion")
        application   = REQUEST.has_key("application")
        report        = REQUEST.has_key("report")
        for UserGroup in UserGroupList:
          self._setUserAllowedModules(UserGroup,users,desicion,application,report)
      
      
  def _createModulesTable(self):
    qdata = self.getDBObj()
    sqlobj = qdata.getsqlobject()
    fields = [{'name':self.modules_userID      ,'type':'varchar(30)', "primary":""},
              {'name':self.modules_users       ,'type':'integer'},
              {'name':self.modules_desicion    ,'type':'integer'},
              {'name':self.modules_application ,'type':'integer'},
              {'name':self.modules_report      ,'type':'integer'}]
    sqlobj.create_table(self.modules_tableID,fields)
    qdata.results_qauth(sqlobj) 
    
    
  def _updateModulesTable(self,username,users,desicion,application,report):
    qdata = self.getDBObj()
    sqlobj = qdata.getsqlobject()
    dic = {self.modules_users       : users,
           self.modules_desicion    : desicion,
           self.modules_application : application,
           self.modules_report      : report
           }
    if self.getUserModules(username,no_recurse=1):
      cond = ( self.modules_userID , username )
      sqlobj.updaterecords(self.modules_tableID,dic,cond=cond,auth=0)
    else:
      dic.update( {self.modules_userID : username} )
      sqlobj.insertrecords(self.modules_tableID,dic)
    qdata.results_qauth(sqlobj)     
    
    
  def _setUserAllowedModules(self,username,users,desicion,application,report):
    self._updateModulesTable(username,users,desicion,application,report)

    
  def _setmode(self,REQUEST):
    buttonname  =  REQUEST.get('buttonname','')
    buttonvalue = REQUEST.get('buttonvalue','')
    
    if buttonname == "auth/modules":
        REQUEST.set("AuthModulesMode",buttonvalue)
        
    elif buttonname == "groups/users":
      REQUEST.set("GroupUserMode",buttonvalue)
      REQUEST.set("user_selected",'')
  
  
    
  def refreshProducts(self,REQUEST):
    username = REQUEST.get('users_selected','')
    user     = self.aq_parent.acl_users.getUser(username)
    parentid = user.getGroups(no_recurse=1)
    
    if self.isGroup(username):
      for user in self.aq_parent.acl_users.getUsers():
        if username in user.getGroups():
          if not user._isGroup:
            self._refreshUserProducts(str(user) )
      self._refreshUserProducts(str("admin") )     
    else:
      self._refreshUserProducts(username)

      
    
  def _auth_action(self,REQUEST):
    AuthModulesMode =  REQUEST.get('AuthModulesMode')
    
    if AuthModulesMode == "auth":
      formframe       =  REQUEST.get('formframe','')

      if formframe == "usermenu":      
        buttonname      = REQUEST.get('buttonname','')
        UserGroupList   = REQUEST.get('ChkUserGroup',[])
        
        if UserGroupList:
          nodelist = REQUEST.get('nodelist',[])
          
          if nodelist:
            auth = REQUEST.get('auth_level')
            mode = REQUEST.get('auth_mode')

            if buttonname == "save":

              for UserGroup in UserGroupList:
                for node in nodelist:
                  self._setAuth(UserGroup,node,auth,mode)
              
            elif buttonname == "delete":
              
              for UserGroup in UserGroupList:
                for node in nodelist:
                  self._deleteAuth(UserGroup,node)
                  
          # end if
          
          elif buttonname == "delete":

            for UserGroup in UserGroupList:
              self._deleteAuth(UserGroup,nodename=None)
              
        # end if   

      # end if 

    # end if 
            
        

  
  def _setAuth(self,username='',nodename='',auth=-1,mode="Read"):
    qdata = self.getDBObj()
    cond  = ((self.auth_userID,username),'and',(self.auth_nodeID,nodename))
    sqlobj = qdata.getsqlobject()
    sqlobj.selectrecords(self.auth_tableID,[self.auth_levelID],cond,auth=0)
    rec = qdata.results_qauth(sqlobj)
    if len(rec):
      sqlobj.updaterecords(self.auth_tableID,
                           {self.auth_levelID:int(auth),self.auth_modeID:str(mode)},
                           cond,0)
    else:
      sqlobj.insertrecords(self.auth_tableID,
                           {self.auth_userID:username,
                            self.auth_nodeID:nodename,
                            self.auth_modeID:str(mode),
                            self.auth_levelID:int(auth)})
    qdata.results_qauth(sqlobj)
    
    
  def _deleteAuth(self,username,nodename=None):
    qdata = self.getDBObj()
    cond  = (self.auth_userID,username)
    if nodename:
      cond  = (cond,'and',(self.auth_nodeID,nodename))
    sqlobj = qdata.getsqlobject()
    sqlobj.deleterecords(self.auth_tableID,cond,auth=0)
    qdata.results_qauth(sqlobj)
    
    

  

InitializeClass(kzUserManagement) 
